/****************************************************************************
*
* (C) 2000-2003 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module         : BIOS.C
* Function       : BIOS API functions
* ---------------------------------------------------------------------------

$Header: BIOS.C, 27, 13.04.2004 15:00:31, Andre Pribil$

*****************************************************************************/


/****************************************************************************/
//includes
/****************************************************************************/

#include <dos.h>
#include <common.h>
#include <BIOS_API.H>

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 BIOS API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces BIOS_API\.H><efont> |
    <tab><linkto>#stdio<linkmid>stdio<linktail><tab>
    <linkto>#Files<linkmid>Files<linktail><tab>
    <linkto>#Ethernet/PPP<linkmid>Ethernet/PPP <linktail><tab>
    <linkto>#IPC@Chip<linkmid>IPC@Chip<linktail><tab>
 @libcat1  stdio   | bios101 | BIOS_kbhit | Test if character is ready on standard input. |
 @libcat1  stdio   | bios100 | BIOS_getch | Get character from standard input, with wait. |
 @libcat1  stdio   | bios10E | BIOS_putch | Output character on standard output. |
 @libcat1  stdio   | bios211 | BIOS_Set_Focus | Select stdio focus. |
 @libcat1  stdio   | bios225 | BIOS_Set_Focus_Hotkey | Set the stdio focus key. |
 @libcat1  stdio   | bios233 | BIOS_Select_Stdio | Specify stdin/stdout channels. |
 @libcat1  stdio   | bios234 | BIOS_Get_Stdio | Check stdin/stdout channels. |
 @libcat1  stdio   | bios235 | BIOS_Install_User_Stdio | Install user stdio handlers. |


 @libcat2  Files   | bios237 | BIOS_Get_File_Sharing | Check the file sharing mode.|
 @libcat2  Files   | bios237a | BIOS_Set_File_Sharing | Enable/Disable file sharing.|
 @libcat2  Files   | bios228 | BIOS_Fast_Findfirst | Fast findfirst.|
 @libcat2  Files   | bios229 | BIOS_Fast_Findnext | Fast findnext.|
 @libcat2  Files   | bios230 | BIOS_Fast_Find_Done | Unlock directory after search.|
 @libcat2  Files   | bios238 | BIOS_Get_File_Name | Get file name by handle.|


 @libcat3  Ethernet/PPP   | bios217 | BIOS_Get_MAC |
        Get MAC address of the Ethernet interface.|
 @libcat3  Ethernet/PPP  | bios219 | BIOS_Config_Server_Level |
        Change level for configuration server.|
 @libcat3  Ethernet/PPP  | bios227 | BIOS_Server_On_Off | Suspend/Resume system servers.|
 @libcat3  Ethernet/PPP  | bios231 | BIOS_Ethernet_State | Detect Ethernet link state.|
 @libcat3  Ethernet/PPP  | bios236 | BIOS_Connection_Handler |
            Install a system server connection handler.|
 @libcat3  Ethernet/PPP  | bios240 | BIOS_Install_UDP_Cfg_Callback |
            Install a UDP Config Server user callback.|
 @libcat3  Ethernet/PPP  | bios250 | BIOS_Get_PPPServer_IP |
            Get the IP address of the PPP server. |
 @libcat3  Ethernet/PPP  | bios255 | BIOS_Get_PPPClient_IP |
            Get the IP address of the PPP client. |


 @libcat4  IPC@Chip   | bios200 | BIOS_Get_Serial_Number |
    Get the serial number of the IPC@CHIP.|
 @libcat4  IPC@Chip   | bios200A | BIOS_Get_Product_Serial_Number |
    Get the serial number of the BECK product.|
 @libcat4  IPC@Chip   | bios200B | BIOS_Get_Hardware_Rev |
    Get the hardware revision number of the IPC@CHIP device.|
 @libcat4  IPC@Chip   | bios200C | BIOS_Get_Product_Hardware_Rev |
    Get the hardware revision number of the BECK product.|
 @libcat4  IPC@Chip   | bios212 | BIOS_Get_Boot_Version | Get bootstrap version number.|
 @libcat4  IPC@Chip   | bios213 | BIOS_Get_RTOS_Version | Get RTOS version.|
 @libcat4  IPC@Chip   | bios222 | BIOS_Version_String | Get RTOS version string.|

 @libcat4  IPC@Chip   | bios207 | BIOS_Execute | Execute a command shell command.|
 @libcat4  IPC@Chip   | bios208 | BIOS_Set_Timer_0x1C | Set timer 0x1C interval.|
 @libcat4  IPC@Chip   | bios209 | BIOS_Set_Timer_0xAF | Set timer 0xAF interval.|
 @libcat4  IPC@Chip   | bios214 | BIOS_Set_Batch_Mode | Set Batch file mode.|
 @libcat4  IPC@Chip   | bios215 | BIOS_Batch_Continue | Continue batch file execution.|
 @libcat4  IPC@Chip   | bios216 | BIOS_Get_Features | Report RTOS features.|
 @libcat4  IPC@Chip   | bios218 | BIOS_Power_Save | Enter low power standby mode.|
 @libcat4  IPC@Chip   | bios218A | BIOS_Power_SaveEx | Make power save mode settings.|
 @libcat4  IPC@Chip   | bios220 | BIOS_Install_Error_Handler |
            Install a user fatal error handler.|
 @libcat4  IPC@Chip   | bios221 | BIOS_Reboot | Reboot the CPU.|
 @libcat4  IPC@Chip   | bios223 | BIOS_Set_Ini_String | Insert an entry into CHIP.INI.|
 @libcat4  IPC@Chip   | bios224 | BIOS_Get_Ini_String | Get an entry from CHIP.INI.|
 @libcat4  IPC@Chip   | bios226 | BIOS_Get_Device_Names | Get the IPC@CHIP device names.|
 @libcat4  IPC@Chip   | bios226A | BIOS_Get_Device_NamesEx | Get the IPC@CHIP device names. (Extended)|
 @libcat4  IPC@Chip   | bios232 | BIOS_Set_Memory_Gap |
            Set a memory gap between DOS programs.|
 @libcat4  IPC@Chip   | bios245 | BIOS_Write_Persistent_Data |
            Writes persistent data into the @CHIPs Flash memory |
 @libcat4  IPC@Chip   | bios246 | BIOS_Read_Persistent_Data |
            Reads persistent data from the @CHIPs Flash memory. |

*/

// Local Data ////////////////////////////////////////////////////////////////
#define BIOS_SWI_1                 (0x10)

// SWI 0x10 services (AH reg)
#define BIOS_GETCHAR               (0x00)
#define BIOS_CHAR_READY            (0x01)
#define BIOS_PUTCHAR               (0x0E)

#define BIOS_SWI_2                 (0xA0)

// SWI 0xA0 services (AH reg)
#define BIOS_GET_SN                (0x00)
#define BIOS_EXECUTE               (0x07)
#define BIOS_SET_TIMER_1C          (0x08)
#define BIOS_SET_TIMER_AF          (0x09)
#define BIOS_SET_IO_FOCUS          (0x11)
#define BIOS_GET_BOOT_VERSION      (0x12)
#define BIOS_GET_RTOS_VERSION      (0x13)
#define BIOS_SET_BATCH_MODE        (0x14)
#define BIOS_BATCH_CONTINUE        (0x15)
#define BIOS_FEATURES              (0x16)
#define BIOS_GET_MAC               (0x17)
#define BIOS_POWER_SAVE            (0x18)
#define BIOS_SERVER_LEVEL          (0x19)
#define BIOS_INSTALL_ERR           (0x20)
#define BIOS_REBOOT                (0x21)
#define BIOS_VERSION_STRING        (0x22)
#define BIOS_SET_CHIP_INI          (0x23)
#define BIOS_GET_CHIP_INI          (0x24)
#define BIOS_FOCUS_HOTKEY          (0x25)
#define BIOS_GET_DEVICE_NAMES      (0x26)
#define BIOS_SERVER_ON_OFF         (0x27)
#define BIOS_FAST_FINDFIRST        (0x28)
#define BIOS_FAST_FINDNEXT         (0x29)
#define BIOS_FAST_FINDDONE         (0x30)
#define BIOS_ETHERNET_STATE        (0x31)
#define BIOS_MEMORY_GAP            (0x32)
#define BIOS_SELECT_STDIO          (0x33)
#define BIOS_GET_STDIO             (0x34)
#define BIOS_INSTALL_USER_STDIO    (0x35)
#define BIOS_INSTALL_CONN_HANDLER  (0x36)
#define BIOS_FILE_SHARING          (0x37)
#define BIOS_GET_FILENAME          (0x38)
#define BIOS_UDP_CALLBACK          (0x40)
#define BIOS_WRITE_PERSISTENT_DATA (0x45)
#define BIOS_READ_PERSISTENT_DATA  (0x46)
#define BIOS_GET_PPPSERVER_IP      (0x50)
#define BIOS_GET_PPPCLIENT_IP      (0x55)

/*************************************************************************/
// Get character from stdio
/*************************************************************************/
#ifdef _bios100_
/*
 @doc DOC_bios100_
 ---------------------------------------------------------------
 Get char from standard input
 ---------------------------------------------------------------
 @clib  BIOS_getch | Get char from standard input, with wait. |
        <cfont>unsigned int BIOS_getch ( void );<efont> |
 @parm  void |  | -- none --
 @rdesc Lower byte:  input character<nl>
        Upper byte:  Source of input - 1: EXT, 2: COM, 4: Telnet, 8: User channel
 @comm  If only the input character is of interest, the return value can be cast
        to a character to suppress the data source indication carried in the
        upper byte.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x00<linkmid>documentation<linktail>
        for more details.
 @Also  bios101.htm |  BIOS_kbhit |
        bios10E.htm |  BIOS_putch |
        bios211.htm |  BIOS_Set_Focus
 @Also  bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios233.htm |  BIOS_Select_Stdio | |
*/
unsigned int BIOS_getch (void)
{
    _AH = BIOS_GETCHAR ;
    asm { int BIOS_SWI_1  } ;

    asm mov ah, dl ; // place source of input in AH

    return _AX;
}
#endif

/*************************************************************************/
// Test if character ready at stdin
/*************************************************************************/
#ifdef _bios101_
/*
 @doc DOC_bios101_
 ---------------------------------------------------------------
 Test if char ready at standard input
 ---------------------------------------------------------------
 @clib  BIOS_kbhit | Test if character is ready at standard input(s). |
        <cfont>unsigned int BIOS_kbhit ( void );<efont> |
 @parm  void |  | -- none --
 @rdesc Boolean: '1' if characters is ready, '0' if no character ready.
 @comm  All devices selected as standard input (e.g. EXT, COM and Telnet)
        are polled for data ready.<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x01<linkmid>documentation<linktail>
        for more details.
 @Also  bios100.htm |  BIOS_getch |
        bios10E.htm |  BIOS_putch|
        bios211.htm |  BIOS_Set_Focus
 @Also  bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios233.htm |  BIOS_Select_Stdio |
        bios234.htm |  BIOS_Get_Stdio
*/
unsigned int BIOS_kbhit (void)
{
    _AH = BIOS_CHAR_READY ;
    asm { int BIOS_SWI_1  } ;
    return _AX ;
}
#endif

/*************************************************************************/
// Output character to standard output.
/*************************************************************************/
#ifdef _bios10E_
/*
 @doc DOC_bios10E_
 ---------------------------------------------------------------
 Output character to standard output.
 ---------------------------------------------------------------
 @clib  BIOS_putch | Output character to standard output(s). |
        <cfont>void BIOS_putch ( unsigned char output );<efont> |
 @parm  unsigned char | output | Character to be output
 @rdesc -- none --
 @comm  This character is output to each device selected for standard output
        (e.g. EXT, COM and Telnet).<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0x100x0E<linkmid>documentation<linktail>
        for more details.
 @Also  bios100.htm |  BIOS_getch |
        bios101.htm |  BIOS_kbhit |
        bios211.htm |  BIOS_Set_Focus
 @Also  bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios233.htm |  BIOS_Select_Stdio |  |
*/
void BIOS_putch (unsigned char output)
{
    _AL = output ;
    _AH = BIOS_PUTCHAR ;
    asm { int BIOS_SWI_1  } ;
}
#endif

/*************************************************************************/
// Get chip serial number
/*************************************************************************/
#ifdef _bios200_
/*
 @doc DOC_bios200_
 ---------------------------------------------------------------
 Get the serial number of the CPU device
 ---------------------------------------------------------------
 @clib  BIOS_Get_Serial_Number | Get the serial number of the IPC@CHIP device. |
        <cfont>unsigned long BIOS_Get_Serial_Number ( void );<efont> |
 @parm  void |  | -- none --
 @rdesc IPC@CHIP serial number
 @comm  The IPC@Chip serial number is a 24 bit value.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x00<linkmid>documentation<linktail>
        for more details.

*/
unsigned long BIOS_Get_Serial_Number (void)
{
    union {

        struct {
            unsigned int  Lower ;
            unsigned int  Upper ;
        } Halfs ;
        unsigned long Whole ;

    }  serial_number ;

    _AH = BIOS_GET_SN ;
    asm { int BIOS_SWI_2  } ;

    serial_number.Halfs.Lower = _AX ;
    serial_number.Halfs.Upper = _BX ;

    return serial_number.Whole ;
}
#endif


/*************************************************************************/
// Get BECK product serial number
/*************************************************************************/
#ifdef _bios200A_
/*
 @doc DOC_bios200A_
 ---------------------------------------------------------------
 Get the serial number of the BECK product
 ---------------------------------------------------------------
 @clib  BIOS_Get_Product_Serial_Number | Get the serial number of the BECK product. |
        <cfont>unsigned long BIOS_Get_Product_Serial_Number ( void );<efont> |
 @parm  void |  | -- none --
 @rdesc BECK product serial number
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x00<linkmid>documentation<linktail>
        for more details.
*/
unsigned long BIOS_Get_Product_Serial_Number (void)
{
    union {

        struct {
            unsigned int  Lower ;
            unsigned int  Upper ;
        } Halfs ;
        unsigned long Whole ;

    }  serial_number ;

    _AH = BIOS_GET_SN ;
    asm { int BIOS_SWI_2  } ;

    serial_number.Halfs.Lower = _CX ;
    serial_number.Halfs.Upper = _DX ;

    return serial_number.Whole ;
}
#endif


/*************************************************************************/
// Get IPC@CHIP hardware revision
/*************************************************************************/
#ifdef _bios200B_
/*
 @doc DOC_bios200B_
 ---------------------------------------------------------------
 Get the hardware revision number of the IPC@CHIP device
 ---------------------------------------------------------------
 @clib  BIOS_Get_Hardware_Rev | Get the hardware revision number
        of the IPC@CHIP device. |
        <cfont>unsigned int BIOS_Get_Hardware_Rev ( void );<efont> |
 @parm  void |  | -- none --
 @rdesc IPC@CHIP hardware revision number
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x00<linkmid>documentation<linktail>
        for more details.
*/
unsigned int BIOS_Get_Hardware_Rev (void)
{
    _AH = BIOS_GET_SN ;
    asm { int BIOS_SWI_2  } ;

    return _DI;
}
#endif


/*************************************************************************/
// Get BECK product hardware revision
/*************************************************************************/
#ifdef _bios200C_
/*
 @doc DOC_bios200C_
 ---------------------------------------------------------------
 Get the hardware revision number of the BECK product
 ---------------------------------------------------------------
 @clib  BIOS_Get_Product_Hardware_Rev | Get the hardware revision number
        of the BECK product. |
        <cfont>unsigned int BIOS_Get_Product_Hardware_Rev ( void );<efont> |
 @parm  void |  | -- none --
 @rdesc BECK product hardware revision number
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x00<linkmid>documentation<linktail>
        for more details.
*/
unsigned int BIOS_Get_Product_Hardware_Rev (void)
{
    _AH = BIOS_GET_SN ;
    asm { int BIOS_SWI_2  } ;

    return _SI;
}
#endif

/*************************************************************************/
// Execute command
/*************************************************************************/
#ifdef _bios207_
/*
 @doc DOC_bios207_
 ---------------------------------------------------------------
 Execute a command shell command
 ---------------------------------------------------------------
 @clib  BIOS_Execute | Execute a command shell command. |
        <cfont>int BIOS_Execute ( const char far *command_line );<efont> |
 @parm  byte | command_line | Pointer to a null terminated command line.
 @rdesc 0   : succes<nl>
        !=0 : on error<nl>
 @comm  This function passes the command string to the command interpreter.<nl>
        <nl>
        Internal commands are processed in the current task, external
        commands (.exe files) are loaded and executed in a new task.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x07<linkmid>documentation<linktail>
        for more details.
*/
int BIOS_Execute (const char far *command_line)
{
    _ES = FP_SEG (command_line) ;
    _DX = FP_OFF (command_line) ;
    _AH = BIOS_EXECUTE ;
    asm { int BIOS_SWI_2  } ;
    return _AX;
}
#endif

/*************************************************************************/
// Set timer 0x1C speed
/*************************************************************************/
#ifdef _bios208_
/*
 @doc DOC_bios208_
 ---------------------------------------------------------------
 Set timer 0x1C interval
 ---------------------------------------------------------------
 @clib  BIOS_Set_Timer_0x1C | Set timer 0x1C interval. |
        <cfont>void BIOS_Set_Timer_0x1C ( unsigned int interval_ms ); <efont> |
 @parm  unsigned int | interval_ms | Timer interval in milliseconds.
 @rdesc -- none --
 @comm  This function defines the interval in milliseconds that timer
        interrupt 0x1C will use..<nl>
        <nl>
        Use <code>setvect(0x1c,your_routine)<codee> to change the interrupt
        vector.<s2> Define your routine as:<nl>
        <nl>
        <tab8><code>void interrupt my_function(void)<codee><nl>
        <nl>
        You must restore the old timer interrupt vector before ending
        the program.<nl>
        <nl>
        Your interrupt routine must be as short as possible without
        any waiting or endless loops.<s2> Avoid the usage of large CLib
        functions like <code>printf<codee>.<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x08<linkmid>documentation<linktail>
        for more details.
 @Also  bios209.htm |  BIOS_Set_Timer_0xAF | | | |
*/
void BIOS_Set_Timer_0x1C (unsigned int interval_ms)
{
    _BX = interval_ms ;
    _AH = BIOS_SET_TIMER_1C ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Set timer 0xAF speed
/*************************************************************************/
#ifdef _bios209_
/*
 @doc DOC_bios209_
 ---------------------------------------------------------------
 Set timer 0xAF interval
 ---------------------------------------------------------------
 @clib  BIOS_Set_Timer_0xAF | Set timer 0xAF interval. |
        <cfont>void BIOS_Set_Timer_0xAF ( unsigned int interval_ms );<efont> |
 @parm  int | interval_ms | Timer interval in milliseconds.
 @rdesc -- none --
 @comm  This function defines the interval in milliseconds that timer
        interrupt 0xAF will use..<nl>
        <nl>
        Use <code>setvect(0xAF,your_routine)<codee> to change the interrupt
        vector.<s2> Define your routine as:<nl>
        <nl>
        <tab8><code>void interrupt my_function(void)<codee><nl>
        <nl>
        You must restore the old timer interrupt vector before ending
        the program.<nl>
        <nl>
        Your interrupt routine must be as short as possible without
        any waiting or endless loops.<s2> Avoid the usage of large CLib
        functions like <code>printf<codee>.<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x09<linkmid>documentation<linktail>
        for more details.
 @Also  bios208.htm |  BIOS_Set_Timer_0x1C | | | |
*/
void BIOS_Set_Timer_0xAF (unsigned int interval_ms)
{
    _BX = interval_ms ;
    _AH = BIOS_SET_TIMER_AF ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Set STDIO focus
/*************************************************************************/
#ifdef _bios211_
/*
 @doc DOC_bios211_
 ---------------------------------------------------------------
 Set STDIO focus
 ---------------------------------------------------------------
 @clib  BIOS_Set_Focus | Set STDIO focus. |
        <cfont>void BIOS_Set_Focus ( IO_FOCUS focus );<efont> |
 @parm  int | focus | Enumeration type:<nl>
        <ul>
        FOCUS_SHELL = 1:  Set focus to command shell<nl>
        FOCUS_APPLICATION = 2:  Set focus to application<nl>
        FOCUS_BOTH = 3: Set focus to both application and command shell<nl>
        <ule>
 @rdesc -- none --
 @comm  This function sets the focus of STDIO to either console, application
        or both.<nl>
        <nl>
        If your application requires input from the user, you should set the
        focus to the application.<s2> You should take care that only one
        application requests input from STDIO.<nl>
        <nl>
        The user can change the focus by using the focus hot key (default
        Ctrl-F).<s2> Changing the focus clears the serial input and output
        queues immediately.<nl>
        <nl><bold>Important<bolde>:<nl>
        <ul>
        All buffered incoming and outgoing characters in the internal serial
        send and receive queues are lost after this call.<nl>
        <ule>
        <nl>
        This function includes a one millisecond sleep.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x11<linkmid>documentation<linktail>
        for more details.
 @related | ../../config.htm#STDIO_FOCUSKEY | Focus | key definition
 @Also  bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios100.htm |  BIOS_getch |
        bios10E.htm |  BIOS_putch
 @Also  bios101.htm |  BIOS_kbhit |
        bios233.htm |  BIOS_Select_Stdio |
        bios234.htm |  BIOS_Get_Stdio
*/
void BIOS_Set_Focus (IO_FOCUS focus)
{
    _AL = (unsigned char) focus ;
    _AH = BIOS_SET_IO_FOCUS ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Get bootstrap version number
/*************************************************************************/
#ifdef _bios212_
/*
 @doc DOC_bios212_
 ---------------------------------------------------------------
 Get bootstrap version number
 ---------------------------------------------------------------
 @clib  BIOS_Get_Boot_Version | Get bootstrap version number. |
        <cfont>unsigned int BIOS_Get_Boot_Version ( void );<efont> |
 @parm   void |  | -- none --
 @rdesc Bootstrap program version number, in BCD format:<nl>
        MSB is major version, LSB is minor version.
 @comm  Example:<nl><ul>
        If the function returns 0x020C, this means that you have version 2.12.
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x12<linkmid>documentation<linktail>
        for more details.
 @Also  bios213.htm |  BIOS_Get_RTOS_Version | | | |
*/
unsigned int BIOS_Get_Boot_Version (void)
{
    _AH = BIOS_GET_BOOT_VERSION ;
    asm { int BIOS_SWI_2  } ;
    return _AX ;
}
#endif

/*************************************************************************/
// Get RTOS version number
/*************************************************************************/
#ifdef _bios213_
/*
 @doc DOC_bios213_
 ---------------------------------------------------------------
 Get RTOS version number
 ---------------------------------------------------------------
 @clib  BIOS_Get_RTOS_Version | Get RTOS version number. |
        <cfont>unsigned int BIOS_Get_RTOS_Version( unsigned int *beta_flag );<efont> |
 @parm  int | beta_flag | Optional output parameter:<s2>
        If non-null, this location will be set as a Boolean to
        indicate if the RTOS is a beta version.<s2>A zero indicates
        that it is not a beta version, non-zero indicates a
        beta version.
 @rdesc RTOS program version number, in split binary format:<nl>
        MSB is binary major version, LSB is binary minor version.
 @comm  Example:<nl><ul>
        If the function returns 0x0043, this means that you have RTOS
        version 0.67.
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x13<linkmid>documentation<linktail>
        for more details.
 @Also  bios212.htm |  BIOS_Get_Boot_Version |
        bios222.htm |  BIOS_Version_String | |
*/
unsigned int BIOS_Get_RTOS_Version (unsigned int *beta_flag)
{
    unsigned int prog_num ;
    _DX = 0 ;               // In case API does not support beta flag
    _AH = BIOS_GET_RTOS_VERSION ;
    asm { int BIOS_SWI_2  } ;
    prog_num = _AX ;
    if (beta_flag != 0)         // Caller wants to see the flag?
    {
        *beta_flag = _DX ;
    }
    return prog_num ;
}
#endif

/*************************************************************************/
// Set Batch file mode
/*************************************************************************/
#ifdef _bios214_
/*
 @doc DOC_bios214_
 ---------------------------------------------------------------
 Set Batch file mode
 ---------------------------------------------------------------
 @clib  BIOS_Set_Batch_Mode | Set Batch file mode. |
        <cfont>void  BIOS_Set_Batch_Mode ( BATCH_MODE mode,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; unsigned int enable_timeout );<efont> |
 @parm  int | mode |
        BATCH_CONCURRENT= 0: <s2> Sets <code>BATCHMODE<codee>=0 = concurrent<nl>
        BATCH_SEQUENTIAL= 1: <s2> Sets <code>BATCHMODE<codee>=1 = sequential<nl>
 @parm  int | enable_timeout | Boolean:<nl>
        0: <s2> Disable the maximum delayed execution time-out of DOS programs<nl>
        1: <s2> Enable the maximum delayed execution time-out of DOS programs in a batchfile,
        if <code>BATCHMODE<codee> = 1 (sequential)<nl>
 @rdesc -- none --
 @comm  This function sets the batch file execution mode of DOS programs
        for either concurrent or sequential execution.<s2> See
        <code>BATCHMODE<codee> initialization
        <linkto>../../config.htm#BATCH_BATCHMODE<linkmid>documentation<linktail>
        for details.<nl>
        <nl>
        The <code>enable_timeout<codee> parameter only applies to
        <code>BATCH_SEQUENTIAL<codee> mode.
        <nl>
        <bold>Important:<bolde><nl>
        <ul>
        If<s1> <code>BATCHMODE=1<codee> take care that every program in your batch
        file which has a successor program either exits
        <linkto>../../dosemu.htm#0x210x4C<linkmid>(int21h 0x4C)<linktail>
        or terminates resident with
        <linkto>../../dosemu.htm#0x210x31<linkmid>int21h 0x31<linktail>.<s2>
        A program which runs forever should call
        <linkto>bios215.htm<linkmid>BIOS_Batch_Continue<linktail> API function,
        which immediately enables further batch file sequencing.<nl>
        <nl>
        By default the maximum delay time for execution of the next listed
        program in the batch file is 15 seconds.<s2>
        If <code>enable_timeout<codee> parameter is set to 0, the successor program
        in a batch file waits forever for execution, if the predecessor program does
        not finish or call <linkto>bios215.htm<linkmid>BIOS_Batch_Continue<linktail>.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x14<linkmid>documentation<linktail>
        for more details.
 @Also  bios215.htm |  BIOS_Batch_Continue | | | |
*/
void BIOS_Set_Batch_Mode (BATCH_MODE mode, unsigned int enable_timeout)
{
    _BX = enable_timeout ;
    _AL = (unsigned char) mode ;
    _AH = BIOS_SET_BATCH_MODE ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Continue Batch file execution
/*************************************************************************/
#ifdef _bios215_
/*
 @doc DOC_bios215_
 ---------------------------------------------------------------
 Continue Batch file execution
 ---------------------------------------------------------------
 @clib  BIOS_Batch_Continue | Continue batch file execution. |
        <cfont>void BIOS_Batch_Continue ( void );<efont> |
 @parm  int |  |  -- none --
 @rdesc -- none --
 @comm  This call allows the  next program listed in a batch file to start execution.<s2>
        This is necessary when the
        <linkto>../../config.htm#BATCH_BATCHMODE<linkmid>BATCHMODE<linktail> is set to
        '1' for sequential execution of programs.<nl>
        <nl>
        This function is implemented by waking up a batch file execution task which
        dispatches any subsequent program listed in the batch file.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x15<linkmid>documentation<linktail>
        for more details.
 @Also  bios214.htm |  BIOS_Set_Batch_Mode | | | |
*/
void BIOS_Batch_Continue (void)
{
    _AH = BIOS_BATCH_CONTINUE ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Get Features
/*************************************************************************/
#ifdef _bios216_
/*
 @doc DOC_bios216_
 ---------------------------------------------------------------
 Get RTOS features
 ---------------------------------------------------------------
 @clib  BIOS_Get_Features | Report RTOS features. |
        <cfont>unsigned long BIOS_Get_Features ( void );<efont> |
 @parm  int |  |  -- none --
 @rdesc Bit field indicating features in this RTOS version with following
        bit assignments:<nl>
        <nl>
        Bit 0:<s2> BGF_ETHERNET   - Ethernet device for TCPIP<nl>
        Bit 1:<s2> BGF_PPP_SERVER - PPP server<nl>
        Bit 2:<s2> BGF_PPP_CLIENT - PPP client<nl>
        Bit 3:<s2> BGF_WEB_SERVER - Web server<nl>
        Bit 4:<s2> BGF_TELNET_SERVER - Telnet server<nl>
        Bit 5:<s2> BGF_FTP_SERVER - FTP server<nl>
        Bit 6:<s2> BGF_TFTP_SERVER - TFTP server<nl>
        Bit 7:<s2> BGF_DHCP_CLIENT - DHCP client<nl>
        ....<nl>
        Bit 16:<s2> BGF_I2C_BUS - I2C-Bus API<nl>
        Bit 17:<s2> BGF_HW_API - Hardware API<nl>
        Bit 18:<s2> BGF_RTOS_API - RTOS API<nl>
        Bit 19:<s2> BGF_PACKET - Packet driver interface for ethernet<nl>
        Bit 20:<s2> BGF_XMODEM - Serial XMODEM filetransfer<nl>
        Bit 21:<s2> BGF_EXT_DISK - External disk interface<nl>
        Bit 22:<s2> BGF_SOFT_SPI - Software SPI API<nl>
        ....<nl>
        Bit 24:<s2> BGF_SNMP_MIB - SNMP MIB variables support (see
        <linkto>../TCPIP/tcp60.htm<linkmid>Get_SNMP_Data<linktail>)<nl>
        Bit 25:<s2> BGF_UDP_CONFIG - UDP config server<nl>
        Bit 26:<s2> BGF_PING_CLIENT - Ping client (see
        <linkto>../TCPIP/tcp75.htm<linkmid>Ping_Open<linktail>)<nl>
        Bit 27:<s2> BGF_DEVICE_DRIVER - TCP/IP device driver<nl>
        Bit 28:<s2> BGF_WEB_PUT - Webfile upload (Webserver PUT method)<nl>
        <nl>
        The mask names stated here (e.g. BGF_PPP_SERVER) are defined in
        BIOS_API.H include file.
 @comm  The bit field returned is coded as so:<nl>
        <nl>
        <ul>
        Bit=0: service or device is not available.<nl>
        Bit=1: service or device is available.<nl>
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x16<linkmid>documentation<linktail>
        for more details.
 @Also  bios213.htm |  BIOS_Get_RTOS_Version | | | |
*/
unsigned long BIOS_Get_Features (void)
{
    union {

        struct {
            unsigned int  AX ;
            unsigned char DL ;
            unsigned char BL ;
        } Parts ;
        unsigned long Whole ;

    }  bitmap ;

    _AH = BIOS_FEATURES ;
    asm { int BIOS_SWI_2  } ;

    bitmap.Parts.AX = _AX ;
    bitmap.Parts.DL = _DL ;
    bitmap.Parts.BL = _BL ;

    return bitmap.Whole ;
}
#endif


/*************************************************************************/
// Get Mac address
/*************************************************************************/
#ifdef _bios217_
/*
 @doc DOC_bios217_
 ---------------------------------------------------------------
 Get Mac address
 ---------------------------------------------------------------
 @clib  BIOS_Get_MAC | Get MAC address of the Ethernet interface. |
        <cfont>void  BIOS_Get_MAC ( unsigned char far *mac );<efont> |
 @parm  int | mac |  Output parameter:<s2>Pointer to a 6 byte memory
        area where the MAC address will be stored
 @rdesc -- none --
 @comm  This function outputs the Ethernet MAC address as a six byte
        array.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x17<linkmid>documentation<linktail>
        for more details.
 @Also  ../tcpip/tcpIPR.htm | Get_IPConfig | | | |
*/
void BIOS_Get_MAC (unsigned char far *mac)
{
    _ES = FP_SEG(mac) ;
    _DX = FP_OFF(mac) ;

    _AH = BIOS_GET_MAC ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Power save
/*************************************************************************/
#ifdef _bios218_
/*
 @doc DOC_bios218_
 ---------------------------------------------------------------
 Power save
 ---------------------------------------------------------------
 @clib  BIOS_Power_Save | Power save. |
        <cfont>void BIOS_Power_Save ( void );<efont> |
 @parm  int |  |  -- none --
 @rdesc -- none --
 @comm  This function slows down the internal timer for
        the RTOS and puts the CPU in a halt mode until the next
        interrupt occurs.<s2>
        Call this function when your program is in idle state.<nl>
        <nl>
        Power savings are marginal since DRAM is used by the CPU.<nl>
        <nl>
        Note:
        <ul>
        This function will affect the internal date/time.<nl>
        Power consumption may differ slightly when
        the date code of the IPC@CHIP is changed.<nl>
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x18<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Power_Save (void)
{
    _AH = BIOS_POWER_SAVE ;
    _DX = 0;
    asm { int BIOS_SWI_2  } ;
}
#endif


/*************************************************************************/
// Power save Exdended
/*************************************************************************/
#ifdef _bios218A_
/*
 @doc DOC_bios218A_
 ---------------------------------------------------------------
 Power save
 ---------------------------------------------------------------
 @clib  BIOS_Power_SaveEx | Power save. Only on SC11/SC13|
        <cfont>int BIOS_Power_SaveEx ( unsigned char mode );<efont> |
 @parm  unsigned char | mode |  Mode<nl>
        <tab>0x01: Restore full speed mode<nl>
        <tab>0x02: Set 1/2 speed mode<nl>
        <tab>0x04: Set 1/4 speed mode (serial baudrates > 57600 will no longer be accurate)<nl>
        <tab>0x80: Enter sleep mode (RTOS 1ms timer and time/date will be affected)<nl>
        <tab>0x81: Wakeup from sleep mode<nl>
        <tab>0x20: Power down Ethernet controller (no Ethernet communication is possible)<nl>
        <tab>0x21: Power up Ethernet controller<nl>
 @rdesc 0 if successful
 @comm  This function slows down the processor clock and puts the CPU
        in a powersave mode until the original mode is restored.<nl>
        Note:
        Power consumption may differ slightly when
        the date code of the IPC@CHIP is changed.<nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x18<linkmid>documentation<linktail>
        for more details.
*/
int BIOS_Power_SaveEx (unsigned char mode)
{
    _AH = BIOS_POWER_SAVE ;
    _DX = 0xABCD;
    _CX = 0x1234;
    _AL = mode;
    asm { int BIOS_SWI_2  } ;
    return _AX;
}
#endif

/*************************************************************************/
// Change level for configuration server
/*************************************************************************/
#ifdef _bios219_
/*
 @doc DOC_bios219_
 ---------------------------------------------------------------
 Change level for configuration server
 ---------------------------------------------------------------
 @clib  BIOS_Config_Server_Level | Change level for configuration server. |
        <cfont>void BIOS_Config_Server_Level ( unsigned int level );<efont> |
 @parm  int | level  |  The supported level
 @rdesc -- none --
 @comm  This function changes the supported level for the configuration server.<s2>
        For a description of the possible levels, refer to
        <linkto>../../config.htm#UDPCFG_LEVEL<linkmid>UDPCFG<linktail> level
        topic.<nl>
        <nl>
        Note that if the level defined in the
        <cfont>CHIP.INI<efont> is 0 (zero),
        the configuration server task is not started and changing the
        supported level has no effect.<s2> To avoid this, use an unlisted
        support level such as 0x1000 in the <cfont>CHIP.INI<efont>.<nl>
        <nl>
        The entry in the <cfont>CHIP.INI<efont> file is not changed by
        this call.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x19<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Config_Server_Level (unsigned int level)
{
    _BX = level ;
    _AH = BIOS_SERVER_LEVEL ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Install a user fatal error handler
/*************************************************************************/
#ifdef _bios220_
/*
 @doc DOC_bios220_
 ---------------------------------------------------------------
 Install a user fatal error handler
 ---------------------------------------------------------------
 @clib  BIOS_Install_Error_Handler | Install a user fatal error handler. |
        <cfont>void BIOS_Install_Error_Handler ( USER_ERROR_HANDLER vector );<efont> |
 @parm  int | vector  |  Far pointer to callback procedure.<s2>Set to zero
        to un-install the handler.
 @rdesc -- none --
 @comm  The user can install an error handler function, that will execute if a
        fatal error occurs in an application or inside of the @CHIP-RTOS.<nl>
        <nl>
        The callback function must be be from:<nl>
        <nl>
        <tab><code>void huge _pascal Err_Handler (unsigned int error_code)<codee>
        <nl><nl>
        The <code>error_code<codee> input parameter can have the following values:<nl>
        <ul>
        1:<s2> Invalid opcode (usually caused by corrupted memory),
        the calling task is suspended<nl>
        2:<s2> Fatal kernel error (usually caused by corrupted memory
        or a task stack overflow)<nl>
        3:<s2> Fatal internal TCPIP error, calling task is supended<nl>
        4:<s2> TCPIP stack reaches memory limit<nl>
        5:<s2> TCPIP memory allocation error<nl>
        6:<s2> Ethernet bus error (hardware defect)<nl>
        7:<s2> Ethernet link error(cable not connected?)<nl>
        8:<s2> Flash write error -> Flash defect (IPC@Chip is no longer usable)<nl>
        9:<s2> Low Memory error -> called if an alloc (system or user) failed<nl>
        <ule>
        In all cases we recommend a reboot with BIOSint 0xA0 0x21 (see below).
        If the flash device has a defect, the IPC@Chip is no longer usable
        <nl>
        <bold>Important:<bolde><nl>
        <ul>
        Do not use any message printing inside your error handler
        if errorcode is 3 or 4 and telnet is part of your stdio.<s2> In this case
        your exit handler would hang inside of the print call.<nl>
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x20<linkmid>documentation<linktail>
        for more details.
 @Also  bios221.htm |  BIOS_Reboot | | | |
*/
void BIOS_Install_Error_Handler (USER_ERROR_HANDLER vector)
{
    _ES = FP_SEG(vector) ;
    _DI = FP_OFF(vector) ;
    _AH = BIOS_INSTALL_ERR ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Reboot the CPU.
/*************************************************************************/
#ifdef _bios221_
/*
 @doc DOC_bios221_
 ---------------------------------------------------------------
 Reboot the CPU.
 ---------------------------------------------------------------
 @clib  BIOS_Reboot | Reboot the CPU. |
        <cfont>void BIOS_Reboot ( void );<efont> |
 @parm  int |  |  -- none --
 @rdesc No return from this function
 @comm  This function works the same as the shell reboot command.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x21<linkmid>documentation<linktail>
        for more details.
 @Also  bios220.htm |  BIOS_Install_Error_Handler | | | |
*/
void BIOS_Reboot (void)
{
    _AH = BIOS_REBOOT ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Get RTOS version string
/*************************************************************************/
#ifdef _bios222_
/*
 @doc DOC_bios222_
 ---------------------------------------------------------------
 Get RTOS version string
 ---------------------------------------------------------------
 @clib  BIOS_Version_String | Get RTOS version string. |
        <cfont>void BIOS_Version_String ( char far *version, int max_len );<efont> |
 @parm  int | version  | Output parameter:<s2>
        Up to <code>max_len<codee> characters of the RTOS's
        version string will be output here.<nl>
 @parm  int | max_len  | One count less than size of buffer at
        <code>version<codee>, to allow space for terminating
        nil character.
 @rdesc -- none --
 @comm  <bold>Caution:<bolde><nl>
        <ul>
        The string at <code>version<codee> must be one greater
        than <code>max_len<codee> to accommodate the string
        terminating nil character which is output after
        the <code>version<codee> string.
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x22<linkmid>documentation<linktail>
        for more details.
 @Also  bios213.htm |  BIOS_Get_RTOS_Version | | | |
*/
void BIOS_Version_String (char far *version, int max_len)
{
    _CX = max_len ;
    _ES = FP_SEG(version) ;
    _DI = FP_OFF(version) ;
    _AH = BIOS_VERSION_STRING ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Insert an entry into CHIP.INI
/*************************************************************************/
#ifdef _bios223_
/*
 @doc DOC_bios223_
 ---------------------------------------------------------------
 Insert an entry into CHIP.INI
 ---------------------------------------------------------------
 @clib  BIOS_Set_Ini_String | Insert an entry into CHIP.INI. |
        <cfont>int BIOS_Set_Ini_String ( const char far *section,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; const char far *item_name,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; const char far *item_text );<efont> |
 @parm  int | section  | Pointer to section string (max. 40 chars)
 @parm  int | item_name  | Pointer to item name (max. 40 chars)
 @parm  int | item_text  | Pointer to item text (max. 128 chars)
 @rdesc 0:<s2>success<nl>
       -1:<s2>Invalid string length
 @comm  <bold>Important<bolde>:<nl>
        <ul>
        The API functions BIOS_Set_Ini_String and
        <linkto>bios224.htm<linkmid>BIOS_Get_Ini_String<linktail>
        are not reentrant.<s2>Do not use these in different tasks or in
        combination with @CHIP-RTOS commands which write to CHIP.INI e.g.
        DHCP.<s2> Avoid race conditions with any other API calls which
        read or write the CHIP.INI file.<nl>
        <ule>
        <nl>Example:<nl>
        @raw
        BIOS_Set_Ini_String ("MY_SECTION", "MY_ITEM", "VALUE_TEXT") ;

        ... produces the following CHIP.INI entry

        [MY_SECTION]
        MY_ITEM=VALUE_TEXT

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x23<linkmid>documentation<linktail>
        for more details.
 @Also  bios224.htm |  BIOS_Get_Chip_Ini | | | |
*/
int BIOS_Set_Ini_String (const char far *section,
                         const char far *item_name,
                         const char far *item_text)
{
    asm { push DS } ;
    _BX = FP_SEG(section) ;
    _SI = FP_OFF(section) ;
    asm { LES DI,item_name } ;
    asm { LDS DX,item_text } ;
    _AH = BIOS_SET_CHIP_INI ;
    asm { int BIOS_SWI_2  } ;
    asm {pop DS } ;
    return _AX ;
}
#endif

/*************************************************************************/
// Get an entry from CHIP.INI
/*************************************************************************/
#ifdef _bios224_
/*
 @doc DOC_bios224_
 ---------------------------------------------------------------
 Get an entry from CHIP.INI
 ---------------------------------------------------------------
 @clib  BIOS_Get_Ini_String | Get an entry from CHIP.INI. |
        <cfont>int BIOS_Get_Ini_String ( const char far *section,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; const char far *item_name,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; char far *item_text,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; unsigned int max_len );<efont> |
 @parm  int | section  | Pointer to section string (max. 40 chars)
 @parm  int | item_name  | Pointer to item name (max. 40 chars)
 @parm  int | item_text  | Output parameter:<s2>Pointer to where
        up to <code>max_len<codee> characters of item text will be written,
        plus a terminating nil character.
 @parm  int | max_len  | Maximum characters to output to
        <code>item_text<codee>, not including terminating nil character.<nl>
        <nl>
        <bold>Caution:<bolde><nl>
        <ul>
        Buffer at <code>item_text<codee> must be <code>max_len<codee> + 1
        size.
        <ule>
 @rdesc 0:<s2>Entry not found<nl>
        non-zero:<s2>Length of the string found
 @comm  <bold>Important<bolde>:<nl>
        <ul>
        The API functions
        <linkto>bios223.htm<linkmid>BIOS_Set_Ini_String<linktail>
        and BIOS_Get_Ini_String
        are not reentrant.<s2>Do not use these in different tasks or in
        combination with @CHIP-RTOS commands which write to CHIP.INI e.g.
        DHCP.<s2> Avoid race conditions with any other API calls which
        read or write the CHIP.INI file.<nl>
        <ule>
        <nl>Example:<nl>
        @raw
        // Declare an output buffer to receive the item's text
        unsigned char item_text[100] ;

        unsigned int text_length =
        BIOS_Get_Ini_String("MY_SECTION", "MY_ITEM",item_text, sizeof(item_text)-1);

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x24<linkmid>documentation<linktail>
        for more details.
 @Also  bios223.htm |  BIOS_Set_Chip_Ini | | | |
*/
int BIOS_Get_Ini_String (const char far *section,
                         const char far *item_name,
                         char far *item_text,
                         unsigned int max_len)
{
    asm { push DS } ;
    _CX = max_len ;
    _BX = FP_SEG(section) ;
    _SI = FP_OFF(section) ;
    asm { LES DI,item_name } ;
    asm { LDS DX,item_text } ;
    _AH = BIOS_GET_CHIP_INI ;
    asm { int BIOS_SWI_2  } ;
    asm {pop DS } ;
    return _AX ;
}
#endif

/*************************************************************************/
// Set the Stdio Focuskey
/*************************************************************************/
#ifdef _bios225_
/*
 @doc DOC_bios225_
 ---------------------------------------------------------------
 Set the Stdio Focus key
 ---------------------------------------------------------------
 @clib  BIOS_Set_Focus_Hotkey | Set the stdio focus key. |
        <cfont>void BIOS_Set_Focus_Hotkey ( unsigned char hotkey );<efont> |
 @parm  int | hotkey  | New focus key code
        in range:<s2> [ 0 .. 254 ]<nl>
 @rdesc -- none --
 @comm  The default hot key is CTRL-F, ASCII 6.<nl>
        <nl>If the focus key is set to zero, the switching of stdio
        from the console is disabled.<nl>
        <nl>At runtime, the hot key (e.g. Ctrl-F) toggles the stdio between
        three modes:<nl><nl>
        <ul>
        Stdio: Shell<s2>  -<s2> Focus set to command shell<nl>
        Stdio: User<s2> -<s2> Focus set to applications<nl>
        Stdio: Both<s2> -<s2> Command shell and applications<nl>
        <ule>
        <nl>
        The focus key is not usable by the command shell or DOS executable.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x25<linkmid>documentation<linktail>
        for more details.
 @related | ../../config.htm#STDIO_FOCUSKEY | Focus | key definition
          in CHIP.INI.
 @Also  bios211.htm |  BIOS_Set_Focus |
        bios100.htm |  BIOS_getch |
        bios10E.htm |  BIOS_putch
 @Also  bios101.htm |  BIOS_kbhit |
        bios233.htm |  BIOS_Select_Stdio |  |
*/
void BIOS_Set_Focus_Hotkey (unsigned char hotkey)
{
    _AL = hotkey ;
    _AH = BIOS_FOCUS_HOTKEY ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Get the IPC@CHIP device names
/*************************************************************************/
#ifdef _bios226_
/*
 @doc DOC_bios226_
 ---------------------------------------------------------------
 Get the IPC@CHIP device names
 ---------------------------------------------------------------
 @clib  BIOS_Get_Device_Names | Get the IPC@CHIP device names. |
        <cfont>void BIOS_Get_Device_Names ( char far **fixed_name,<nl>
        <tab><tab><tab><tab><tab><tab><tab> char far **configured_name );<efont> |
 @parm  int | fixed_name  | Output paramter:<s2> Pointer to a
        string pointer which will receive a pointer to the
        fixed device name stored in the IPC@CHIP flash.
 @parm  int | configured_name  | Output paramter:<s2> Pointer to a
        string pointer which will receive a pointer to the device
        name configured in CHIP.INI.
 @rdesc -- none --
 @comm  You cannot write to the string at <code>fixed_name<codee>, because
        it is located in flash memory.<nl>
        <nl>
        The string at <code>configured_name<codee> is up to 20 characters
        long.<nl><nl>
        Both returned strings are zero terminated.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x26<linkmid>documentation<linktail>
        for more details.
 @related  | ../../config.htm#DEVICENAME | Device name | definition
        in CHIP.INI file
*/
void BIOS_Get_Device_Names (char far **fixed_name,
                            char far **configured_name)
{
    WORD bx, es ;
    _AH = BIOS_GET_DEVICE_NAMES ;
    asm { int BIOS_SWI_2  } ;
    bx = _BX ;              // Compiler needs this one.
    es = _ES;               // Compiler needs this one.
    *configured_name = MK_FP(bx, _SI) ;
    *fixed_name = MK_FP(es, _DI) ;
}
#endif


/*************************************************************************/
// Get the IPC@CHIP device names (Extended)
/*************************************************************************/
#ifdef _bios226A_
/*
 @doc DOC_bios226A_
 ---------------------------------------------------------------
 Get the IPC@CHIP device names (Extended)
 ---------------------------------------------------------------
 @clib  BIOS_Get_Device_NamesEx | Get the IPC@CHIP device names. (Extended) |
        <cfont>void BIOS_Get_Device_NamesEx ( char far **fixed_name,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; char far **configured_name,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; char far **beckproduct_name );<efont> |
 @parm  int | fixed_name  | Output paramter:<s2> Pointer to a
        string pointer which will receive a pointer to the
        fixed device name stored in the IPC@CHIP flash.
 @parm  int | configured_name  | Output paramter:<s2> Pointer to a
        string pointer which will receive a pointer to the device
        name configured in CHIP.INI.
 @parm  int | beckproduct_name  | Output paramter:<s2> Pointer to a
        string pointer which will receive a pointer to the
        fixed BECK product device name stored in the IPC@CHIP flash.
 @rdesc -- none --
 @comm  You cannot write to the string at <code>fixed_name<codee>, because
        it is located in flash memory.<nl>
        <nl>
        The string at <code>configured_name<codee> is up to 20 characters
        long.<nl><nl>
        All returned strings are zero terminated.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x26<linkmid>documentation<linktail>
        for more details.
 @related  | ../../config.htm#DEVICENAME | Device name | definition
        in CHIP.INI file
*/
void BIOS_Get_Device_NamesEx (char far **fixed_name,
                              char far **configured_name,
                              char far **beckproduct_name)
{
    WORD bx,es ;
    _AH = BIOS_GET_DEVICE_NAMES ;
    asm { int BIOS_SWI_2  } ;
    bx = _BX ;              // Compiler needs this one.
    es = _ES;               // Compiler needs this one.
    *configured_name = MK_FP(bx, _SI) ;
    *fixed_name = MK_FP(es, _DI) ;
    *beckproduct_name = MK_FP(_DX, _CX) ;
}
#endif

/*************************************************************************/
// Suspend/Resume System Servers
/*************************************************************************/
#ifdef _bios227_
/*
 @doc DOC_bios227_
 ---------------------------------------------------------------
 Suspend/Resume System Servers
 ---------------------------------------------------------------
 @clib  BIOS_Server_On_Off | Suspend/Resume system servers. |
        <cfont>void BIOS_Server_On_Off ( SERVER_SPEC which_server,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; unsigned int activate );<efont> |
 @parm   int | which_server  | Enumerator:<nl>
      <ul>
        SS_FTP_SERVER = 0, SS_TELNET_SERVER = 1, SS_WEB_SERVER = 2
      <ule>
 @parm int | activate  | Boolean:<s2> 0:<s2> Resume,<s2> 1:<s2> Suspend.
 @rdesc -- none --
 @comm This function may be used to enable a server that was
       disabled at startup due to a CHIP.INI entry.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x27<linkmid>documentation<linktail>
        for more details.
 @related  | ../../config.htm#FTP_ENABLE | FTP_ENABLE | definition
       in CHIP.INI file
 @related  | ../../config.htm#TELNET_ENABLE | TELNET_ENABLE | definition
       in CHIP.INI file
 @related  | ../../config.htm#WEB_ENABLE | WEB_ENABLE | definition
       in CHIP.INI file
*/
void BIOS_Server_On_Off (SERVER_SPEC which_server, unsigned int activate)
{
    _BX = which_server ;
    _AL = (unsigned char) activate ;
    _AH = BIOS_SERVER_ON_OFF ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Fast findfirst
/*************************************************************************/
#ifdef _bios228_
/*
 @doc DOC_bios228_
 ---------------------------------------------------------------
 Fast findfirst
 ---------------------------------------------------------------
 @clib  BIOS_Fast_Findfirst | Provide a faster access to the file
        systems directories. |
        <cfont>int BIOS_Fast_Findfirst ( char far *filename,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; FILE_FIND far *find,<nl>
        <tab><tab><tab><tab><tab><tab>&nbsp; unsigned int attributes );<efont> |
 @parm   int | filename  | Pointer to null terminated ASCII file name pattern
         for search.
 @parm   int | find  | Pointer to a FILE_FIND structure.
 @parm   int | attributes  | File attributes
 @rdesc -1:<s2>Findfirst is already active<nl>
         0:<s2>No file found<nl>
         1:<s2>Success
 @comm   This function and the
         <linkto>bios229.htm<linkmid>BIOS_Fast_Findnext<linktail>
         provide a faster
         findfirst/next access than do the DOS compatible functions available on
         software interrupt
         <linkto>../../DOSEMU.HTM#0x210x4E<linkmid>0x21<linktail>.<nl>
         <nl>
         These functions work in a similiar way to the DOS SWI 0x21 findfirst/next
         functions.<s2> BIOS_Fast_Findfirst must be called first.<s2> Then the
         BIOS_Fast_Findnext is used repeatedly as required to iterate through
         all the files in a directory matching the specified <code>filename<codee>
         pattern and  <code>attributes<codee>.<nl>
         <nl>
         The directory being searched will be locked for exclusive access
         for the calling task.<nl>The
         <linkto>bios230.htm<linkmid>BIOS_Fast_Find_Done<linktail> function
         <bold>must
         be called to release this directory lock.<bolde>.<nl>
         <nl>
         These functions are not reentrant.<s2>The user must provide semaphore
         protection if these functions are to be called from different tasks.<nl>
         <nl>
         Note:
         <ul>
         This function will convert the <code>filename<codee> string to
         all upper case.<nl>
         <nl>
         <ule>
         The FILE_FIND data structure format is:<nl>
         @raw
         typedef struct filefind
         {
           char    filename[12];               // Null terminated filename
           char    fileext[4];                 // and extension
           unsigned short fileattr;            // MS-DOS file attributes
           short int     reserved;            // Reserved

           struct tag_filetimestamp
           {
             // Date = ((year - 80) shl 9) or (month shl 5)) or day
             unsigned short int filedate;
             // Time = (hour shl 11) or (min shl 5)) or (sec / 2)
             unsigned short int filetime;
           } filetimestamp;                        // Time & date last modified

           unsigned long   filesize ;              // File size
           char            private_field[180];     // Reserved, used internal

         } FILE_FIND ;
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x28<linkmid>documentation<linktail>
        for more details.
 @Also  bios229.htm |  BIOS_Fast_Findnext |
        bios230.htm |  BIOS_Fast_Find_Done | |
*/
int BIOS_Fast_Findfirst (char far *filename,
                         FILE_FIND far *find,
                         unsigned int attributes)
{
    _CX = attributes ;
    _BX = FP_SEG(filename) ;
    _SI = FP_OFF(filename) ;
    _ES = FP_SEG(find) ;
    _DI = FP_OFF(find) ;

    _AH = BIOS_FAST_FINDFIRST ;
    asm { int BIOS_SWI_2  } ;

    _AX |= _DX ;
    return _AX ;
}
#endif

/*************************************************************************/
// Fast findnext
/*************************************************************************/
#ifdef _bios229_
/*
 @doc DOC_bios229_
 ---------------------------------------------------------------
 Fast findnext
 ---------------------------------------------------------------
 @clib  BIOS_Fast_Findnext | Continue iteration of files in a directory. |
        <cfont>int BIOS_Fast_Findnext ( FILE_FIND far *find );<efont> |
 @parm  int | find | Pointer to the FILE_FIND structure used at the
        <linkto>bios228.htm<linkmid>BIOS_Fast_Findfirst<linktail> call.
 @rdesc -1:<s2>Failure, <linkto>bios228.htm<linkmid>BIOS_Fast_Findfirst<linktail>
        not called before<nl>
        0:<s2>No file found<nl>
        1:<s2>Success
 @comm  This function continues the iteration of files found in a directory.
        <s2>The directory search process must be started with the
        <linkto>bios228.htm<linkmid>BIOS_Fast_Findfirst<linktail> function.
        <nl>
        <nl>
        These functions are not reentrant.<s2>The user must provide semaphore
        protection if these functions are to be called from different tasks.<nl>
        <nl>
        <bold>Important:<bolde>
        <ul>
        The <linkto>bios230.htm<linkmid>BIOS_Fast_Find_Done<linktail> function
        must be called when finished with the iteration, to unlock the
        directory.<nl>
        <ule>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x29<linkmid>documentation<linktail>
        for more details.
 @Also  bios228.htm |  BIOS_Fast_Findfirst |
        bios230.htm |  BIOS_Fast_Find_Done | |
*/
int BIOS_Fast_Findnext (FILE_FIND far *find)
{
    _ES = FP_SEG(find) ;
    _DI = FP_OFF(find) ;

    _AH = BIOS_FAST_FINDNEXT ;
    asm { int BIOS_SWI_2  } ;

    _AX |= _DX ;
    return _AX ;
}
#endif

/*************************************************************************/
// Fast find done
/*************************************************************************/
#ifdef _bios230_
/*
 @doc DOC_bios230_
 ---------------------------------------------------------------
 Fast find done
 ---------------------------------------------------------------
 @clib  BIOS_Fast_Find_Done| Unlock the directory following Findfirst/next
        operation.  |
        <cfont>int BIOS_Fast_Find_Done ( FILE_FIND far *find ); <efont> |
 @parm  int | find  | Pointer to the FILE_FIND structure used at the
        <linkto>bios228.htm<linkmid>BIOS_Fast_Findfirst<linktail> call.
 @rdesc  0:<s2>Successful<nl>
         -1:<s2>Findfirst not active
 @comm  The function unlocks the directory being searched by the
        <linkto>bios228.htm<linkmid>BIOS_Fast_Findfirst<linktail>
        and <linkto>bios229.htm<linkmid>BIOS_Fast_Findnext<linktail>
        functions.<s2>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x30<linkmid>documentation<linktail>
        for more details.
 @Also  bios228.htm |  BIOS_Fast_Findfirst |
        bios229.htm |  BIOS_Fast_Findnext | |
*/
int BIOS_Fast_Find_Done (FILE_FIND far *find)
{
    _ES = FP_SEG(find) ;
    _DI = FP_OFF(find) ;

    _AH = BIOS_FAST_FINDDONE ;
    asm { int BIOS_SWI_2  } ;

    return _DX ;
}
#endif

/*************************************************************************/
// Detect Ethernet link state
/*************************************************************************/
#ifdef _bios231_
/*
 @doc DOC_bios231_
 ---------------------------------------------------------------
 Detect Ethernet link state
 ---------------------------------------------------------------
 @clib  BIOS_Ethernet_State| Detect Ethernet link state.  |
        <cfont>unsigned int BIOS_Ethernet_State ( unsigned int *init_error,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab>&nbsp;&nbsp; unsigned int *phy_state );<efont> |
 @parm  int | init_error  | non-zero indicates initialization or reset procedure
        of ethernet device failed.
 @parm  int | phy_state  | SC13 only: phy status<nl>
        Bit 14 <tab><tab> 1 = Link Not detected<nl>
        Bit 07 <tab><tab> 1 = 100Base-TX mode, 0=10Base-T mode<nl>
        Bit 06 <tab><tab> 1 = Device in Full Duplex mode
 @rdesc non-zero indicates have no Ethernet link.
 @comm  If return value is non-zero (no Ethernet link), this
        may be due to the network cable not being connected.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x31<linkmid>documentation<linktail>
        for more details.
*/
unsigned int BIOS_Ethernet_State (unsigned int *init_error,
                                  unsigned int *phy_state)
{
    _AH = BIOS_ETHERNET_STATE ;
    asm { int BIOS_SWI_2  } ;

    *init_error = _DX;
    *phy_state  = _CX;

    return _AX;
}
#endif

/*************************************************************************/
// Set a memory gap between DOS programs
/*************************************************************************/
#ifdef _bios232_
/*
 @doc DOC_bios232_
 ---------------------------------------------------------------
 Set a memory gap between DOS programs
 ---------------------------------------------------------------
 @clib  BIOS_Set_Memory_Gap | Set the memory gap used between the loaded
        DOS programs.  |
        <cfont>int BIOS_Set_Memory_Gap ( unsigned int paragraphs );<efont> |
 @parm  int | paragraphs  | Size of memory gap in 16 byte paragraphs:<s2>
        range [0 .. 2048]
 @rdesc 0:<s2>Successful<nl>
        -1:<s2>Failure, invalid value in <code>paragraphs<codee><nl>
 @comm  This function sets the size of the memory gap that will be used between
        subsequently loaded DOS programs.<s2>This gap serves as a memory reserve.<nl>
        <nl>
        Some programs compiled with Borland C 5.02 try to increase the
        size of their program memory block at runtime before, for
        example, opening a file with Borland C-Library function fopen.<s2>
        (The programs calls int21h
        <linkto>../../DOSEMU.HTM#0x210x4A<linkmid>0x4A<linktail>.<s2>
        This happens down inside the
        C-Library fopen function and is not directly visible to the application programmer.)<s2>

        This memory resize call fails if another program has been loaded after
        the one making the call due to now there is no memory space
        left for increasing the memory size of the first program.<s2> The program
        returns from fopen with  an error.<s2>
        The global program variable errno is set to value 8 (not enough memory).<s2>
        To prevent this error the size of a memory gap between two loaded programs can
        be controlled.<s2>
        The value is defined in terms of memory paragraphs (1 paragraph = 16 bytes).<s2>
        This value can also defined in the
        <linkto>../../config.htm#DOSLOADERMEMGAP<linkmid>CHIP.INI<linktail> file.<nl>
        <nl>
        This strategy could fail in the cases where programs are terminated and
        are started again.<nl>
        <nl>
        It is usually not necessary to set this entry, if the application does not show
        the described error.<s2>Only if a C-Library function call sets errno to 8,
        should this value be defined.<s2> We recommend in
        case of this error, a memory gap value of 128 paragraphs (2048 Bytes).<s2>

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x32<linkmid>documentation<linktail>
        for more details.
*/
int BIOS_Set_Memory_Gap (unsigned int paragraphs)
{
    _BX = paragraphs ;

    _AH = BIOS_MEMORY_GAP ;
    asm { int BIOS_SWI_2  } ;

    return _AX ;
}
#endif

/*************************************************************************/
//Specify the stdin/stdout channels
/*************************************************************************/
#ifdef _bios233_
/*
 @doc DOC_bios233_
 ---------------------------------------------------------------
 Specify the stdin/stdout channels
 ---------------------------------------------------------------
 @clib  BIOS_Select_Stdio | Specify the stdin / stdout channels.  |
        <cfont>int BIOS_Select_Stdio ( unsigned int ports, unsigned char direction );<efont> |
 @parm  int | ports  | Bit field specifying the channels
        using a combination of following masks:<nl>
        <nl>
        <ul>
        B0:<s2>STDIO_EXT - EXT serial port<nl>
        B1:<s2>STDIO_COM - COM serial port<nl>
        B2:<s2>STDIO_TELNET - Telnet<nl>
        B3:<s2>STDIO_USER - User port<nl>
        <ule>
 @parm  int | direction  | Bit field specifying
        either stdin, stdout or both using a
        combination of following masks<nl><nl>
        <ul>
        B0:<s2>SET_STDOUT<nl>
        B1:<s2>SET_STDIN<nl>
        <ule>
 @rdesc  0:<s2>Successful<nl>
        -1:<s2>Invalid <code>paragraphs<codee><nl>
 @comm  The bit masks are defined in BIOS_API.H include file.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x33<linkmid>documentation<linktail>
        for more details.
 @Also  bios234.htm |  BIOS_Get_Stdio  |
        bios101.htm |  BIOS_kbhit |
        bios10E.htm |  BIOS_putch
 @Also  bios211.htm |  BIOS_Set_Focus |
        bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios100.htm |  BIOS_getch
 @Also  bios235.htm |  BIOS_Install_User_Stdio | | | |
*/
int BIOS_Select_Stdio (unsigned int ports, unsigned char direction)
{
    _BX = ports ;

    _AL = direction ;
    _AH = BIOS_SELECT_STDIO ;
    asm { int BIOS_SWI_2  } ;

    return _AX ;
}
#endif

/*************************************************************************/
// Get the current stdin/stdout channels
/*************************************************************************/
#ifdef _bios234_
/*
 @doc DOC_bios234_
 ---------------------------------------------------------------
 Get the the stdin/stdout channels
 ---------------------------------------------------------------
 @clib  BIOS_Get_Stdio | Get the stdin / stdout channels setting.  |
        <cfont>unsigned int BIOS_Get_Stdio ( void );<efont> |
 @parm  int |   | -- none --
 @rdesc Two bit fields, each one byte.<s2>
        LSB specifies the stdin, MSB specifies the stdout.<s2>
 @comm  Both bit fields returned (MSB for stdout, LSB for
        stdin) are coded as follows:<nl>
        <nl>
        <ul>
        B0:<s2>STDIO_EXT - EXT serial port<nl>
        B1:<s2>STDIO_COM - COM serial port<nl>
        B2:<s2>STDIO_TELNET - Telnet<nl>
        B3:<s2>STDIO_USER - User port<nl>
        <ule>
        <nl>
        The bit masks are defined in BIOS_API.H include file.
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x34<linkmid>documentation<linktail>
        for more details.
 @Also  bios233.htm |  BIOS_Select_Stdio |
        bios101.htm |  BIOS_kbhit |
        bios10E.htm |  BIOS_putch
 @Also  bios211.htm |  BIOS_Set_Focus |
        bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios100.htm |  BIOS_getch
*/
unsigned int BIOS_Get_Stdio (void)
{
    _AH = BIOS_GET_STDIO ;
    asm { int BIOS_SWI_2  } ;

    asm mov ah, bl // move STDOUT to high byte
    asm mov al, cl // move STDIN  to low  byte

    return _AX;
}
#endif

/*************************************************************************/
// Install user specific stdio handlers
/*************************************************************************/
#ifdef _bios235_
/*
 @doc DOC_bios235_
 ---------------------------------------------------------------
 Install user specific stdio handlers
 ---------------------------------------------------------------
 @clib  BIOS_Install_User_Stdio | Install user specific stdio handlers.  |
        <cfont>void BIOS_Install_User_Stdio ( const USER_STDIO_FUNCS far *vectors );<efont> |
 @parm  int | vectors  | Pointer to a USER_STDIO_FUNCS type data
        structure prepared by caller.<s2>Set this parameter to null to
        uninstall the user's device handlers.
 @rdesc -- none --
 @comm  This function installs user specific stdio channel handlers, allowing
        the user to connect their own input/output device such as a display
        or keyboard to the IPC@CHIP.<s2> A TCP application similar to Telnet
        would be another example.<nl>
        <nl>
        The user must implement four functions in their application to cover
        reading and writing characters from/to their new stdin/stdout device.<s2>
        After installing these functions with this API call and then setting stdin
        and/or stdout to the new user channel with the
        <linkto>bios233.htm<linkmid>BIOS_Select_Stdio<linktail><s2> API,
        the @Chip-RTOS will call these user functions at each stdin and
        stdout operation.<nl>
        <nl>

        The following type definitions can be found in BIOS_API.H header
        file:<nl>
        <nl>
        typedef int  huge (far *USER_KBHIT) (void);<nl>
        typedef void huge(far *USER_PUTCH) (char chr);<nl>
        typedef void huge(far *USER_PUTSTR) (const char far * pch, int n);<nl>
        typedef int huge (far *USER_GETCH) (void);<nl>
        <nl><nl>
        typedef struct tag_user_stdio<nl>
        {<nl>
        <tab><tab>   USER_KBHIT<s2>  user_kbhit;<nl>
        <tab><tab>   USER_GETCH<s2>  user_getch;<nl>
        <tab><tab>   USER_PUTCH<s2>  user_putch;<nl>
        <tab><tab>   USER_PUTSTR     user_putstr;<nl>
        } USER_STDIO_FUNCS;<nl><nl>

        <nl>
        Functions to be implemented by the user:<nl><nl>
        // user_kbhit returns a 1 if a character is available, 0 if not.<nl>
        int  huge user_kbhit(void);<nl>
        <nl>
        // user_getch reads a char from stdin (waits if none available)<nl>
        int  huge user_getch(void);<nl>
        <nl>
        // user_putch writes a single character to stdout<nl>
        void huge user_putch(char chr);<tab>  //write a single char to stdout<nl>
        <nl>
        // user_putstr writes a string of n characters to stdout<nl>
        void huge user_putstr(const char far * pch, int n);<nl>
        <nl>
        The user must set the vectors to this set of four callback functions in the
        USER_STDIO_FUNCS structure referenced by the <code>vectors<codee> input
        parameter.<nl>
        <nl>
        The user handlers are uninstalled by calling this API with the
        <code>vectors<codee> parameter set to zero.<nl>
        <nl>
        <bold>Important<bolde>: <nl>
        <ul>
        1. The USER_STDIO_FUNCS data structure must be static.<s2>
        The @Chip-RTOS does not make a copy of this information, but instead
        references this provided data structure during operation.<nl>
        2. If your application exits, don't forget to uninstall your stdio handler
        by calling this API call with <code>vectors<codee> parameter set to zero.<nl>
        3. Do not call your stdio functions from within your application,
        these functions must only be called from the @Chip-RTOS.<nl>
        <ule>
        <nl>
        Example (Borland C):<nl>
        @raw
        USER_STDIO_FUNCS user_stdio_funcs ;    // global variable

        // implementations of user's stdio functions
        int  huge my_kbhit(void)
        {
           //........
        }

        int huge my_getch(void)
        {
           //........
        }

        void huge my_putch(char chr)
        {
           //........
        }

        void huge my_putstr (const char * pch, int n)
        {
           //.......
        }

        void install_my_stdio_channel (void)
        {
           user_stdio_funcs.user_kbhit  = my_kbhit;
           user_stdio_funcs.user_getch  = my_getch;
           user_stdio_funcs.user_putch  = my_putch;
           user_stdio_funcs.user_putstr = my_putstr;

           BIOS_Install_User_Stdio ( &user_stdio ) ;
           BIOS_Select_Stdio ( STDIO_COM | STDIO_TELNET | STDIO_USER, // ports
                   SET_STDOUT | SET_STDIN) ;              // direction
        }

        void remove_my_stdio_channel (void)
        {
           BIOS_Select_Stdio ( STDIO_COM | STDIO_TELNET,              // ports
                   SET_STDOUT | SET_STDIN) ;              // direction

           BIOS_Install_User_Stdio ( 0 ) ;             // un-install
        }

        int main (void)
        {
           //......
           install_my_stdio_channel();
           //....

           // at the end of the program
           remove_my_stdio_channel():
        }

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x35<linkmid>documentation<linktail>
        for more details.
 @Also  bios233.htm |  BIOS_Select_Stdio |
        bios101.htm |  BIOS_kbhit |
        bios10E.htm |  BIOS_putch
 @Also  bios211.htm |  BIOS_Set_Focus |
        bios225.htm |  BIOS_Set_Focus_Hotkey |
        bios100.htm |  BIOS_getch
 @Also  bios234.htm |  BIOS_Get_Stdio | | | |
*/
void BIOS_Install_User_Stdio (const USER_STDIO_FUNCS far *vectors)
{
    _ES = FP_SEG(vectors) ;
    _DI = FP_OFF(vectors) ;

    _AH = BIOS_INSTALL_USER_STDIO ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Install a System Server Connection Handler function
/*************************************************************************/
#ifdef _bios236_
/*
 @doc DOC_bios236_
 ---------------------------------------------------------------
 Install a System Server Connection Handler function
 ---------------------------------------------------------------
 @clib  BIOS_Connection_Handler | Install a system server
        connection handler function.  |
        <cfont>int BIOS_Connection_Handler ( CONN_HANDLER handler,<nl>
        <tab><tab><tab><tab><tab><tab><tab>&nbsp; SERVER_SPEC which_server );<efont> |
 @parm  int | handler  | Vector to callback function
 @parm  int | which_server  | Enumerator:<nl>
        <ul>
        SS_FTP_SERVER = 0, SS_TELNET_SERVER = 1, SS_WEB_SERVER = 2
        <ule>
 @rdesc 0:<s2>Successf<nl>
        -1:<s2>Invalid <code>which_server<codee> parameter<nl>
 @comm  A connection handler function must be declared in the following way:<nl><nl>
        <code>int huge UserConnectionHandler( struct sockaddr_in far *sockptr );<codee><nl><nl>

        The connection handler will be called if a client establishes a connection
        to the server (FTP, WEB, Telnet).<s2> The handler could read the IP Address
        and the port number from the
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>sockaddr_in<linktail> structure.<s2>
        The TCP/IP API function
        <linkto>../TCPIP/tcp11.htm<linkmid>InetToAscii<linktail> might be of use here.<nl>
        <nl>
        If the connection handler returns zero the connection will be established.<s2>
        If it returns a non-zero value, the connection will be abort.<nl>
        <nl>
        Example usage:<nl>
        <nl>
        <ul>
        The implemented handler function could check the source IP address
        (Clients IP), compare this IP with an application internal list of
        allowed IP addresses and reject the connection by returning a non-zero
        value if the source IP is not in the list.<nl><nl>
        <ule>

        A connection handler can be removed by calling BIOS_Connection_Handler
        with a null pointer.

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x36<linkmid>documentation<linktail>
        for more details.
 @Also  ../TCPIP/tcp11.htm |  InetToAscii  | | | |
*/
int BIOS_Connection_Handler (CONN_HANDLER handler, SERVER_SPEC which_server)
{
    _BX = which_server ;
    _ES = FP_SEG(handler) ;
    _DI = FP_OFF(handler) ;

    _AH = BIOS_INSTALL_CONN_HANDLER ;
    asm { int BIOS_SWI_2  } ;

    return _AX ;
}
#endif


/*************************************************************************/
// Get the current file sharing mode
/*************************************************************************/
#ifdef _bios237_
/*
 @doc DOC_bios237_
 ---------------------------------------------------------------
 Get the current file sharing mode
 ---------------------------------------------------------------
 @clib  BIOS_Get_File_Sharing_Mode | Get the current file sharing mode.  |
        <cfont>int BIOS_Get_File_Sharing_Mode ( void );<efont> |
 @parm  int |  | -- none --
 @rdesc sharing mode:<s2>Non-zero if sharing mode is enabled
 @comm  By default file sharing is disabled.<s2> This has the effect, that
        a file which is opened for write access can't be opened a second time for
        read or write access.<s2>
        To avoid this security feature you can enable file sharing.<s2> This
        can also be done with the CHIP.INI entry
        <linkto>../../config.htm#DEVICE_FILESHARING<linkmid>FILESHARING<linktail>.
        <nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x37<linkmid>documentation<linktail>
        for more details.
 @Also  bios237a.htm | BIOS_Set_File_Sharing_Mode  | | | |
*/
int BIOS_Get_File_Sharing_Mode (void)
{
    _AL = 1 ;                   // Get mode.
    _AH = BIOS_FILE_SHARING ;
    asm { int BIOS_SWI_2  } ;

    return _AX ;
}
#endif

/*************************************************************************/
// Enable/Disable file sharing
/*************************************************************************/
#ifdef _bios237a_
/*
 @doc DOC_bios237a_
 ---------------------------------------------------------------
 Enable/Disable file sharing
 ---------------------------------------------------------------
 @clib  BIOS_Set_File_Sharing_Mode | Get the current file sharing mode.  |
        <cfont>void BIOS_Set_File_Sharing_Mode ( unsigned int enable_sharing );<efont> |
 @parm  int | enable_sharing  | Boolean:<s2>Set non-zero to enable file
        sharing, or zero to disable file sharing.
 @rdesc -- none --
 @comm  By default file sharing is disabled.<s2> This has the effect, that
        a file which is opened for write access can't be opened a second time for
        read or write access.<s2>
        To avoid this security feature you can enable file sharing.<s2> This
        can also be done with the CHIP.INI entry
        <linkto>../../config.htm#DEVICE_FILESHARING<linkmid>FILESHARING<linktail>.
        <nl>
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x37<linkmid>documentation<linktail>
        for more details.
 @Also  bios237.htm |  BIOS_Get_File_Sharing_Mode  | | | |
*/
void BIOS_Set_File_Sharing_Mode (unsigned int enable_sharing)
{
    _BX = enable_sharing ;

    _AL = 0 ;                   // Set mode.
    _AH = BIOS_FILE_SHARING ;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Get file name by handle
/*************************************************************************/
#ifdef _bios238_
/*
 @doc DOC_bios238_
 ---------------------------------------------------------------
 Get file name by handle
 ---------------------------------------------------------------
 @clib  BIOS_Get_File_Name | Get file name by handle.  |
        <cfont>int BIOS_Get_File_Name ( char far * filename, unsigned int handle );<efont> |
 @parm  int | filename  | Output parameter:<s2> Pointer to 13
        character buffer which will receive the nil terminated
        file name associated with the provided handle.
 @parm  int | handle  | File handle.
 @rdesc 0:<s2> Success<nl>
       -1:<s2> Invalid file handle
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x38<linkmid>documentation<linktail>
        for more details.
*/
int BIOS_Get_File_Name (char far * filename, unsigned int handle)
{
    asm{
         mov cx,handle
         les bx,filename
         mov ah, BIOS_GET_FILENAME
         int BIOS_SWI_2
    }
    return _AX;
}
#endif

/*************************************************************************/
// Install a UDP Config Server User Callback function
/*************************************************************************/
#ifdef _bios240_
/*
 @doc DOC_bios240_
 ---------------------------------------------------------------
 Install a UDP Config Server User Callback function
 ---------------------------------------------------------------
 @clib  BIOS_Install_UDP_Cfg_Callback | Install a UDP Config Server
        user callback function.  |
        <cfont>void BIOS_Install_UDP_Cfg_Callback ( UDP_CFG_CALLBACK callback );<efont> |
 @parm  int | callback  | Vector to callback function
 @rdesc -- none --
 @comm  This function installs a user callback function for
        the UDP Config Server.<s2> This function will be invoked when a UDP
        configuration request with command number 06 is received.<s2>
        This allows you to implement your own UDP Config sub protocol and
        commands.<nl>
        <nl>
        The UDP Cfg callback function must have the following form:<nl>
        <code><nl>
        <tab>void huge MyUdpCfgSrvCB( struct UdpCfgSrv_UserCBInfo far *infoPtr );<nl>
        <nl> <codee>
        The UDP Cfg callback function is passed an argument with
        information about the UDP Cfg Request and its requester.<s2>
        This data structure, defined in
        BIOS_API.H header file, is repeated below.<s2>The
        <linkto>../../tcpstruc.htm#sockaddr_in<linkmid>sockaddr_in<linktail>
        structure definition can be found in TCPIPAPI.H<nl>
        <nl>
        If the callback function sets a pointer in the <code>dataPtr<codee>
        member of the <code>UdpCfgSrv_UserCBInfo<codee> structure, the UDP Config will
        be sent the referenced data using the <code>dataLength<codee> member of the
        structure for the byte count.<s2> If this pointer is set to null or
        the <code>dataLength<codee> field is set to 0 then no data will be sent back to
        the requester.<nl>
        <nl>
        To remove an installed callback function, call this API function
        with a null pointer.<nl>
        <nl>
        For more information on the UDP Config Server and its protocol have
        a look at the UDP Config Server description available on our website.<nl>
        <nl>
        Note:<s2> The amount data sent and received is limited to 300 bytes maximum.

       @raw
       typedef struct UdpCfgSrv_UserCBInfo
       {
         int    length ;                         // Length of this structure
         struct sockaddr_in far *fromAddrPtr;    // Sender address pointer
         int    udpCfgSD;                        // Socket descriptor from UDP Config Server Socket
         char   far *dataPtr;                    // Data of Request package
         unsigned   dataLength;                  // Length of request package

       } UDPCFGSRV_USERCBINFO ;

 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x40<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Install_UDP_Cfg_Callback (UDP_CFG_CALLBACK callback)
{
    _ES = FP_SEG(callback) ;
    _DI = FP_OFF(callback) ;

    _AH = BIOS_UDP_CALLBACK ;
    asm { int BIOS_SWI_2  } ;
}
#endif


/*************************************************************************/
// Write Persistent user data
/*************************************************************************/
#ifdef _bios245_
/*
 @doc DOC_bios245_
 ---------------------------------------------------------------
 Write Persistent User Data
 ---------------------------------------------------------------
 @clib  BIOS_Write_Persistent_Data | Write Persistent User Data.  |
        <cfont>void BIOS_Write_Persistent_Data ( const void far *data,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab>&nbsp; unsigned char size );<efont> |
 @parm  int | data  | data buffer which contains data to write
 @parm  int | size  | number of bytes to write
 @rdesc none
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x45<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Write_Persistent_Data(const void far *data, unsigned char size)
{
    _ES = FP_SEG(data);
    _SI = FP_OFF(data);
    _CL = size;
    _AH = BIOS_WRITE_PERSISTENT_DATA;
    asm { int BIOS_SWI_2  } ;
}
#endif


/*************************************************************************/
// Read Persistent user data
/*************************************************************************/
#ifdef _bios246_
/*
 @doc DOC_bios246_
 ---------------------------------------------------------------
 Read Persistent User Data
 ---------------------------------------------------------------
 @clib  BIOS_Read_Persistent_Data | Read Persistent User Data.  |
        <cfont>void BIOS_Read_Persistent_Data ( void far *data,<nl>
        <tab><tab><tab><tab><tab><tab><tab><tab> unsigned char size );<efont> |
 @parm  int | data  | data buffer where read data should be stored
 @parm  int | size  | number of bytes to read
 @rdesc none
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x46<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Read_Persistent_Data(void far *data, unsigned char size)
{
    _ES = FP_SEG(data);
    _DI = FP_OFF(data);
    _CL = size;
    _AH = BIOS_READ_PERSISTENT_DATA;
    asm { int BIOS_SWI_2  } ;
}
#endif


/*************************************************************************/
// Get IP address of the PPP server
/*************************************************************************/
#ifdef _bios250_
/*
 @doc DOC_bios250_
 ---------------------------------------------------------------
 Get IP address of the PPP server
 ---------------------------------------------------------------
 @clib  BIOS_Get_PPPServer_IP | Get the IP address of the PPP server.  |
        <cfont>void BIOS_Get_PPPServer_IP ( char far *ipaddr );<efont> |
 @parm  char far *| ipaddr  | Pointer to a 16 byte memory area where the IP
        address is to be stored as a null terminated string. If no PPP IP
        address is set (e.g. no link is established) the function return an
        empty string.
 @rdesc none
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x50<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Get_PPPServer_IP(char far *ipaddr)
{
    _ES = FP_SEG(ipaddr);
    _DX = FP_OFF(ipaddr);
    _AH = BIOS_GET_PPPSERVER_IP;
    asm { int BIOS_SWI_2  } ;
}
#endif

/*************************************************************************/
// Get IP address of the PPP client
/*************************************************************************/
#ifdef _bios255_
/*
 @doc DOC_bios255_
 ---------------------------------------------------------------
 Get IP address of the PPP client
 ---------------------------------------------------------------
 @clib  BIOS_Get_PPPClient_IP | Get the IP address of the PPP client.  |
        <cfont>void BIOS_Get_PPPClient_IP ( char far *ipaddr );<efont> |
 @parm  char far *| ipaddr  | Pointer to a 16 byte memory area where the IP
        address is to be stored as a null terminated string. If no PPP IP
        address is set (e.g. no link is established) the function return an
        empty string.
 @rdesc none
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../biosints.htm#0xA00x55<linkmid>documentation<linktail>
        for more details.
*/
void BIOS_Get_PPPClient_IP(char far *ipaddr)
{
    _ES = FP_SEG(ipaddr);
    _DX = FP_OFF(ipaddr);
    _AH = BIOS_GET_PPPCLIENT_IP;
    asm { int BIOS_SWI_2  } ;
}
#endif



// End BIOS.C ////////////////////////

