/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     httpcli.c
* Function:   HTTP client
*
*******************************************************************************
* Written By Ernest Schloesser
* Copyright BECK IPC GmbH
*
* Extended by ralph hofmann
* Copyright rh-Technik ralph hofmann
*
* $Header: httpcli.c, 1, 01.04.2004 16:20:15, Markus Bartat$
*
******************************************************************************/

/*
 Usage:
 httpGet(char *server, char *page, char *result, int maxlen, char *dns)
 with: server = [user:pass@]servername[:port]
                servername may be a resolvable hostname or a dotted-decimal IP-address
                the "server" string might be changed by this function.
                so if you still need your original string, give this function a copy of it !
                if an authorization is given, httpGet always inserts it in the HTTP request
                 but does not check the servers reaction on an unauthorized request.
       page   = [path/][filename]   no leading slash !
       result = pointer where received data is to be stored
       maxlen = maximum number of bytes to receive (remember your buffer size !)
       dns    = pointer to a string with the IP-address of the DNS to use for
                                                 domain-name resolving
       httpGet returns the number of bytes received.
                the terminating zero is not counted,
                so this return value cannot extend (maxlen-1).
*/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib>
#include "httpcli.h"
#include "base64.h"
#include "dns.h"
#include "clib.h"

/******************************************************************************
* Defines
******************************************************************************/
#define HTTPCLI_RCVBUFSIZE        512
#define MAX_ARG_CMDLEN            128
#define MAX_BASE64_ENCODE_BUFSIZE 500

/******************************************************************************
* global
******************************************************************************/
static char    request        [MAX_HTTP_REQ_BUFLEN + 1];
static char    uri_buf        [MAX_ARG_CMDLEN];
static char    ReceiveBuffer  [HTTPCLI_RCVBUFSIZE];
static char    Base64EncodeBuf[MAX_BASE64_ENCODE_BUFSIZE];

/******************************************************************************
* HTTP Get   not reentrant!!!
  Parameters:
  char * request_line: The HTTP request
  char * request_buf : User provided storage for the received page, zero terminated
  int    max_len     : Max. length of storage request_buf
  char * dns         : Optional, IP address of dns server
  Return >=0, number  of received bytes stored at request_bufsult
          <0, error
******************************************************************************/
int httpGet( char *request_line, char *request_buf, int maxlen, char *dns )
{
   unsigned long ipHttpServer,ttl;
   char ipHttpServerIPStr[16];

   int sd, error,rcvd;
   char  *page_ptr, *help_ptr, *uri_ptr;
   unsigned int httpport, length, cnt = 0;
   unsigned int auth_length=0;

   help_ptr = strchr( request_line, '/' ); //find end of URI ('/')
   if ( help_ptr )
   {
      length = (unsigned int)(help_ptr - request_line);
      page_ptr = help_ptr;  //point to requested page
   }
   else
   {
      return HTTP_INVALID_REQ;
   }
   strncpy( uri_buf, request_line, length);
   help_ptr  = &uri_buf[length];  //point to the end of URI and terminate with zero
   *help_ptr = 0;


   help_ptr = strchr( uri_buf, '@' ); //request line contains user and passw?
   if (help_ptr )   //the uri buffer contains a '@' => Authorization required!!!
   {

      *help_ptr = 0;           // divide the combined string into authorization and address
      uri_ptr = help_ptr+1 ;   //uri_ptr points to the address string only
      auth_length = base64encode((unsigned char *)uri_buf, (unsigned char *)Base64EncodeBuf,
                                 strlen(uri_buf),MAX_BASE64_ENCODE_BUFSIZE);
   }
   else
   {
     uri_ptr = uri_buf;
   }

   //check the server-string (without possible authentication) for a specified http-port
   help_ptr = strchr( uri_ptr, ':' ); // is there a ':' in the server ?
   if ( help_ptr )
   {
      *help_ptr = 0; // cut the server string here
      sscanf( help_ptr + 1, "%u", &httpport );  // and read the port
      printf( "\r\n\nPort-Identifier found at position %u in server string.\n",
              (unsigned) (help_ptr - uri_ptr));
   }
   else
   {
      httpport = 80;
   }

   //is the servername an IP-Address ?
   if ( 0!= inet_addr( uri_ptr, &ipHttpServer ) )
   {
      //no, get the Address from DNS
      strcpy(ipHttpServerIPStr,"0.0.0.0");  //Preset IPAddr
      if(1!= gethostbyname( dns, uri_ptr, &ttl, &ipHttpServer, ipHttpServerIPStr))
      {
         return HTTP_DNS_FAILED;
      }
   }
   else
   {
      strcpy( ipHttpServerIPStr, uri_ptr);
   }

   sprintf( request, "GET %s HTTP/1.1\r\n"
                     "User-Agent: HTTP Client IPC@CHIP\r\n"
                     "Host: %s\r\n"
                     "Connection: Keep-Alive\r\n",
                     page_ptr, uri_ptr);
   if ( auth_length )
   {
      sprintf(request + strlen(request), "Authorization: Basic %s\r\n", Base64EncodeBuf);
   }
   strcat( request, "\r\n" );

   sd = tcp_connect( ipHttpServerIPStr, 0, httpport, &error );

   if ( sd <= 0 )
   {
      printf( "\r\n\nHTTP-Client Error: %d opening connection to %s port %u.",
              error, ipHttpServerIPStr, httpport );
      return 0;
   }

   if ( send( sd, request, strlen( request ), 0, &error ) < 0 )
   {
      printf( "\r\n\nHTTP-Client Error: 0x%04X sending request", error );
      goto EndTransfer;
   }

   help_ptr = request_buf;  //incoming data is to be stored here
   do
   {
      rcvd = recv( sd, ReceiveBuffer, HTTPCLI_RCVBUFSIZE, MSG_TIMEOUT, 10000L, &error );
      if ( error == 0 )
      {
         if ( cnt + rcvd > maxlen - 1 )
         {
            rcvd=( maxlen - 1 ) - cnt;  // reduce number of bytes to copy
            printf( "\r\n\nHTTP-Client Message: didn't load whole page because of to small memory.\n" );
         }

         memcpy( help_ptr, ReceiveBuffer, rcvd);
         help_ptr += rcvd;    // ptr within buffer
         cnt += rcvd;    // number of bytes received
      }
      else
      if ( ( rcvd != -1 ) && ( error != 235 ) )
      {
         printf( "\r\n\nHTTP-Client: Receiving page terminated with error 0x%04X\n",
                 error);
      }
   }
   while ( ( ! error ) && ( help_ptr < ( request_buf + maxlen - 1 ) ) );

   *help_ptr = 0;   // Terminate the received-page-string

EndTransfer:
   if ( closesocket( sd, &error ) < 0 )
   {
      printf( "\r\n\nHTTP-Client Error: %d closing connection to %s port %u.\n",
              error, ipHttpServerIPStr, httpport );
   }
   return cnt;
}
//------------------------------------------------------------------------------
// End of file

