/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     retentiv.c
* Function:   Example of using the hardware API with non-volatile data.
*
*             Shows how important data can be secured at black-out
*             with the help of the NMI.
*             On BCxxx 2kB of data can be saved.
*
*******************************************************************************
*
* $Header: retentiv.c, 2, 31.03.2004 11:15:57, Andre Pribil$
*
******************************************************************************/

/******************************************************************************
* Defines
******************************************************************************/
#define ANZ1 100
#define ANZ2  50
#define MAX_RETENTIVE_AREAS 2  // used by hwapi.h

/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include "clib.h"

/******************************************************************************
* Variables
******************************************************************************/
Remop rop;

unsigned int  var1[ANZ1];
struct type_var2 {
 unsigned int  val1;
 unsigned char val2;
} var2[ANZ2];

/******************************************************************************
* NMI Handler
******************************************************************************/
void huge nmi_handler (void)
{
  // Save retentive operators
  hal_save_non_volatile_data();
}

/******************************************************************************
* Main
******************************************************************************/
void main (void)
{
  int i, init;

  printf("\nTesting non-volatile functions of the IPC@CHIP hardware API");
  printf("\nThis example works only with an external NMI circuit (not on the DK40)");

  // Focus = App
  BIOS_Set_Focus(FOCUS_APPLICATION);

  // Init remop structure
  rop.entries = MAX_RETENTIVE_AREAS;
  rop.segment = FP_SEG (var1);

  // var1 should be saved
  rop.x[0].offs     = FP_OFF (var1);
  rop.x[0].size     = sizeof (var1);
  rop.x[0].maxsize  = 0; // not used
  rop.x[0].elemsize = sizeof (unsigned int); // 1 element
  rop.x[0].distance = sizeof (unsigned int); // no gap between the elements

  // var2.val1 should be saved
  rop.x[1].offs     = FP_OFF (&var2[0].val1); // only val1 should be retentive
  rop.x[1].size     = sizeof (unsigned int)*ANZ2;
  rop.x[1].maxsize  = 0; // not used
  rop.x[1].elemsize = sizeof (unsigned int);
  rop.x[1].distance = sizeof (struct type_var2);

  // Install ISR
  hal_install_isr(15, 1, nmi_handler);

  printf("\nInit/Restore retentive operators");

  // Init/Restore retentive operators
  if (hal_init_non_volatile_data (&rop) < 0)
  {
    printf("\nInit failed!!!");

    // Deinstall NMI handler
    hal_install_isr(15, 0, NULL);

    // Focus = Both
    BIOS_Set_Focus(FOCUS_BOTH);

    return;
  }

  //get last reboot reason
  printf("\r\nLast reboot reason: %d", hal_get_reboot_reason());

  // Compare contents of variables
  init = 0;
  for (i=0; i<ANZ1; i++)
  {
    if (var1[i] != i)
    {
      init = 1;
      printf("\nVar1: %d ", var1[i]);
      var1[i] = i;
    }
  }

  for (i=0; i<ANZ2; i++)
  {
    if (var2[i].val1 != i)
    {
      init = 1;
      printf("\nVar2: %d ", var2[i].val1);
      var2[i].val1 = i;
    }
  }

  if (init)
  {
    printf("\nRetentive operators are NOT identical!");
    printf("\nRetentive operators are now initialized!");
  }
  else
  {
    printf("\nRetentive operators read successfully!");
  }

  printf("\n\nPress a key to exit program and write data now");
  printf("\nor switch power off to write data with NMI");

  while (1)
  {
    if (kbhit())
    {
      getch();

      printf("\nSave data now!");

      // Save retentive operators
      hal_save_non_volatile_data();

      printf("\nDeinstall NMI handler");

      // Deinstall NMI handler
      hal_install_isr(15, 0, NULL);

      // Focus = Both
      BIOS_Set_Focus(FOCUS_BOTH);

      break;
    }
  }

  printf("\nDone");
}