/****************************************************************************
*
* (C) 2004 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Garbenheimerstr. 36-38
*  D-35578 Wetzlar
*
*  Phone : (49)-6441-905-0
*  Fax   : (49)-6441-905-245
*
* ---------------------------------------------------------------------------
* Module      :  snmpv1.h

* Function    :  Protoypes/constants/typedefs, used at snmpv1.c
*******************************************************************************
*
* $Header: snmpv1.h, 1, 15.03.2004 10:29:19, Markus Bartat$
*
*******************************************************************************
* THIS SOFTWARE IS PROVIDED BY BECK IPC GmbH ``AS IS'' AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* ---------------------------------------------------------------------------
*/
#ifndef    _SNMPV1_H_
   #define _SNMPV1_H_

   #include <DKxx.H>

   #include <typedef.h>


   #define MAX_NUMBER_BINDINGS           4

   #define MAX_NUMBER_TRAP_BINDINGS      4

   #define SNMP_PORT                   161
   #define SNMP_TRAP_PORT              162

   // The maximum accepted packet length
   #define SNMP_MAX_MSG_LENGTH        1000

   #define COMMUNITY_STRING_LENGTH      40

   //SNMP versions
   #define SNMP_VERSION_1                0


   #define MAX_OID_LENGTH               50
   #define MAX_OID_VALUE_LENGTH        127

   #define MAX_LENGTH_SNMP_STRING      128
   #define MAX_LEN_REQUEST              20


   //Dynamic mib-ii table sizes
   #define MAX_DEV             4   //max. number of  device interface
   #define MAX_ARP            16   //max. number of  arp cache entries
   #define MAX_ROUTE           5   //max. number of  routing cache entries
   #define MAX_IP_ADDR         5   //max. number of  device interface
   #define MAX_SOCKETS        16   //max. number of open sockets

   #define MAX_IFACE_OID_LEN   (64 - BASE_OID_STRLEN)   //max. length of an interface table oid string
   #define MAX_AT_OID_LEN      (96 - BASE_OID_STRLEN)   //max. length of an AT table oid string
   #define MAX_RT_OID_LEN      (64 - BASE_OID_STRLEN)   //max. length of an RT table oid string
   #define MAX_IPNTM_OID_LEN   (96 - BASE_OID_STRLEN)  //max. length of an IPNetToMedia table oid string
   #define MAX_UDP_OID_LEN     (96 - BASE_OID_STRLEN)   //max. length of an udp table oid string
   #define MAX_TCP_OID_LEN     (128- BASE_OID_STRLEN)  //max. length of a  tcp table oid string
   #define MAX_IPADDR_OID_LEN  (64 - BASE_OID_STRLEN)

   #define INTERFACE_UP                  1
   #define INTERFACE_DOWN                2
   #define INTERFACE_TEST                3

   //SNMP Head structure
   typedef struct  {
       unsigned char bVersion;                          // SNMP Version
       char bCommunityString[COMMUNITY_STRING_LENGTH];  // Community String
   } tSNMPHead;

   //ProcessDataUnit Head
   typedef struct  {
       unsigned char bType;
       unsigned char bRequestNumber[MAX_LEN_REQUEST];
       unsigned int wErrorStatus;
       unsigned int wErrorNumber;
       unsigned int wNumberOfBindings;
   } tPDUHead;

   //Binding
   typedef struct  {
       unsigned char    abOIDBinding[MAX_OID_LENGTH+1];
       unsigned char    abValueBinding[MAX_OID_VALUE_LENGTH];
   } tPDUBinding;

   //Trap Head
   typedef struct  {
       unsigned char bPDUType;
       unsigned char bEnterpriseMib[MAX_OID_LENGTH+1];
       unsigned char bAgentIPAddress[4];
       unsigned int wStandardTrapType;
       unsigned int wSpecificTrapType;
       unsigned long dwTimeStamp;
       unsigned int wNumberOfBindings;
   } tTrapPDUHead;

   //SNMP message structure
   typedef struct  {
       tSNMPHead   tSNMPHeadData;
       tPDUHead    tPUDDataHead;
       tPDUBinding tPUDBindings[MAX_NUMBER_BINDINGS];
   } tSNMPMessage;

   //SNMP trap structure
   typedef struct  {
       tSNMPHead       tSNMPHeadData;
       tTrapPDUHead    tTrapDataHead;
       tPDUBinding     tPUDBindings[MAX_NUMBER_TRAP_BINDINGS];
   } tSNMPTrap;


  //SNMP counter variables
  struct mib_snmp {
    long    snmpInPkts;
    long    snmpOutPkts;
    long    snmpInBadVersions;
    long    snmpInBadCommunityNames;
    long    snmpInBadCommunityUses;
    long    snmpInASNParseErrs;
    long    snmpInTooBigs;
    long    snmpInNoSuchNames;
    long    snmpInBadValues;
    long    snmpInReadOnlys;
    long    snmpInGenErrs;
    long    snmpInTotalReqVars;
    long    snmpInTotalSetVars;
    long    snmpInGetRequests;
    long    snmpInGetNexts;
    long    snmpInSetRequests;
    long    snmpInGetResponses;
    long    snmpInTraps;
    long    snmpOutTooBigs;
    long    snmpOutNoSuchNames;
    long    snmpOutBadValues;
    long    snmpOutGenErrs;
    long    snmpOutGetRequests;
    long    snmpOutGetNexts;
    long    snmpOutSetRequests;
    long    snmpOutGetResponses;
    long    snmpOutTraps;
    long    snmpEnableAuthTraps;
 };
/**************************************************************************/
//Prototypes
/**************************************************************************/
   int init_snmp(void);
   int get_interface_status(void);
   int set_interface_status(int iNewStatus);
   int send_SNMP_trap(unsigned char bTrapNumber, unsigned char bSecTrapNumber);
   #ifdef DKXX_SNMP_DEMO
   void  set_dkxx_bit(unsigned char bit, unsigned char val /* 0 or 1 */);
   unsigned char  get_dkxx_bit(unsigned char bit);
   #endif
   int build_dynamic_mib_tables(void);
/**************************************************************************/
//External decl.
/**************************************************************************/
  extern int  mib_semaphoreID; /*semaphoreID*/
/**************************************************************************/
#endif /*_SNMPV1_H_*/

