/******************************************************************************
*
* (c) 2004 by BECK IPC GmbH
*
* BECK IPC GmbH
* Garbenheimerstr. 36-38
* D-35578 Wetzlar
*
* Phone: (49)-6441-905-0
* Fax:   (49)-6441-905-245
*
*******************************************************************************
*
* Disclaimer: This program is an example and should be used as such.
*             If you wish to use this program or parts of it in your application,
*             you must validate the code yourself. BECK IPC GmbH can not be held
*             responsible for the correct functioning or coding of this example.
*
* Module:     tcpclie.c
* Function:   This program demonstrates the usage of the TCP/IP-API of the IPC@Chip.
*             It sends some data to an echo-server and prints out what it gets back.
* Notes:      Use this in combination with tcpserv.
*             tcpclie expects the IP-address of the host with the echo-server as
*             command line paraeter.
*
*******************************************************************************
*
* $Header: TCPCLIE.C, 3, 31.03.2004 11:16:04, Andre Pribil$
*
******************************************************************************/


/******************************************************************************
* Includes
******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "clib.h"



/******************************************************************************
* Constants
******************************************************************************/
#define TM_PORT_ECHO                      7    /*well known echo port*/
#define TM_CLIENTPORT_ECHO              1234   /*well known echo port*/
#define TM_TCPECHOBUF_CLIENT_SENDSIZE    20
#define TM_TCPECHOBUF_CLIENT_RECVSIZE    20

/*
  these switches are only for debugging:
  comment these switches, if you haved finished your application
  because the print routines takes very much processing time
*/
//#define TCP_ECHO_CLIENT_DEBUG
#define PRINT_DATA



/******************************************************************************
* Global variables
******************************************************************************/
char sendbuf[TM_TCPECHOBUF_CLIENT_SENDSIZE ];
char recvbuf[TM_TCPECHOBUF_CLIENT_RECVSIZE ];

char destIPStr[17];
unsigned long destIPAddr;



/******************************************************************************
* main()
******************************************************************************/
int main(int argc, char *argv[])
{

   struct sockaddr_in   addr;

   int sd;
   int i;
   int error;
   int retval;


   strcpy(destIPStr,"1.1.1.1");
   //get the destination ip from argv
   if(argc==2)
   {
    if(strlen(argv[1]) < 17)
      strcpy(destIPStr,argv[1]);
   }
   else
   {
     printf("\r\nTCPClient: Missing or invalid argument\r\n");

   }

   printf("\r\nTCPClient:  Dest. IP: %s ,  Port %d\r\n",destIPStr,TM_PORT_ECHO);
   #ifdef TCP_ECHO_CLIENT_DEBUG
   printf("\r\nOpen socket");
   #endif
   //API call open socket
   retval = opensocket( SOCK_STREAM, &error );

   if(retval == API_ERROR)
   {
      printf("\r\nTCPClient: Socket open failed: %d", error);
      goto TCP_CLIENT_DONE;
   }
   else
   {
      sd = retval;
   }

   /**********************************************************************/
   //init the sendbuffer
   /**********************************************************************/
   for(i=0;i<TM_TCPECHOBUF_CLIENT_SENDSIZE;i++)
   {
      sendbuf[i]='a';
   }

   /**********************************************************************/
   //API-Call connect, establish a connection to the specified TCP server
   /**********************************************************************/
   //convert server  port to correct byte order
   addr.sin_family =  PF_INET;
   retval = htons( TM_PORT_ECHO );
   addr.sin_port =  retval;
   //convert DestIPString
   retval = inet_addr( (char *)destIPStr, &destIPAddr );

   if(retval == API_ERROR)
   {
      goto TCP_CLIENT_DONE;
   }
   else
   {
     addr.sin_addr.s_addr =  destIPAddr;
   }

   //call connect
   retval = connect( sd, &addr, &error );

   if(retval == API_ERROR)
   {
     if(error==261)  //connection refused from server host
     {
      printf("\r\nTCPClient: Connection refused from %s",destIPStr);
     }
     else
     {
        printf("\r\nTCPClient:Socket connect failed: %d",error);
     }
      goto TCP_CLIENT_DONE;
   }

   printf("\r\nTCPClient connected with %s\r\n",destIPStr);


   while(1)   //loop forever
   {
       /**************************************************************/
       //send some data to the specified echoserver
       /**************************************************************/

      //send
      retval = send( sd, (char *) sendbuf, TM_TCPECHOBUF_CLIENT_SENDSIZE, 0, &error );

      if(retval == API_ERROR)
      {
         #ifdef TCP_ECHO_CLIENT_DEBUG
          printf("\r\nTCPClient: Senderror %d",error);
         #endif

         goto TCP_CLIENT_DONE;
      }

      #ifdef PRINT_DATA
       printf("\r\nTCPClient:\r\nSending :\r\n");
       for(i=0;i<TM_TCPECHOBUF_CLIENT_SENDSIZE;i++)
       {
          printf("%02X ",(char)sendbuf[i]);
       }
      #endif

       /*****************************************************************/
       //Sleep a little bit
       /*****************************************************************/
       api_sleep(300);


       /*******************************************************************/
       //Wait for an answer from the specified echoserver, call recv
       //To prevent a socketoverrun, we read all available data from the socket
        /*******************************************************************/
        do
        {

             //recv
             retval = recv( sd, (char *) recvbuf, TM_TCPECHOBUF_CLIENT_RECVSIZE,
                            MSG_TIMEOUT, 5000L, &error );

             if(retval == API_ERROR)
             {
                #ifdef TCP_ECHO_CLIENT_DEBUG
                 printf("\r\nTCPclient: Recverror %d",error);
                #endif
              goto TCP_CLIENT_DONE;
             }
             else
             {
                if( retval > 0)    //data received
                {
                #ifdef PRINT_DATA
                   printf("\r\nTCPClient\r\nReceived:\r\n");
                   for(i=0;i<retval;i++)
                    {
                      printf("%02X ",(char)recvbuf[i]);
                    }//for(i=0;i<outregs.x.ax;i++)
                #endif
              }
              else
              {
                #ifdef TCP_ECHO_CLIENT_DEBUG
                  printf("\r\nTCPclient: Nothing received");
                  #endif
              }
            }//elseif(retval == API_ERROR)
            /*****************************************************************/
            //Check, if there is more data available at the socket ?
            /*****************************************************************/
            retval = GetWaitingBytes( sd, &error );

      }//while data available
      while((retval != API_ERROR && retval > 0) );




      //modify the sendbuffer for the next sendjob
      for(i=0;i<TM_TCPECHOBUF_CLIENT_SENDSIZE;i++)
      {
        if(sendbuf[i]<'z')
        {
          sendbuf[i]++;
        }
        else
        {
          sendbuf[i]='a';
        }
      }

   }//while(1)   //loop forever

   /***********************************/
   //Closing connection should not happen, if connection is established
   /***********************************/

TCP_CLIENT_DONE:
   //close socket
   printf("\r\nTCPClient: Closing connection\r\n");
   retval = closesocket( sd, &error );

   #ifdef TCP_ECHO_CLIENT_DEBUG
   if(retval==API_ERROR)
   {
       printf("\r\nTCPClient: Socket close failed: %d",error);
   }
   #endif

   return 0;

}



// End of file
